import typing as t
from datetime import datetime

from bs4 import BeautifulSoup

from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.lever.warehouse import (
    LeverJobWarehouse,
    LeverProfileWarehouse,
)
from hrflow_connectors.core import (
    ActionName,
    ActionType,
    BaseActionParameters,
    Connector,
    ConnectorAction,
    ConnectorType,
    WorkflowType,
)


def extract_job_sections(contents: list) -> str:
    lists = contents.get("lists", [])
    sections = []
    for item in lists:
        content = item["content"]
        content_text = html_to_plain_text(content)
        section = dict(
            name=item.get("text", None),
            title=item.get("text", None),
            description=content_text,
        )
        if section:
            sections.append(section)
    return sections


def html_to_plain_text(html: str) -> str:
    if not html:
        return None
    soup = BeautifulSoup(html, "html.parser")
    plain_text = soup.get_text(separator="\n")
    return plain_text


def get_job_tags(lever_job: dict) -> list:
    tags = []

    tags.append(
        {
            "name": "lever_confidential",
            "value": (
                "False"
                if lever_job.get("confidentiality", "") == "non-confidential"
                else "True"
            ),
        }
    )

    categories = lever_job.get("categories", {})
    tags.extend(
        {"name": f"lever_{tag_name}", "value": tag_value}
        for tag_name, tag_value in categories.items()
    )

    additional_tags = lever_job.get("tags", [])
    tags.extend(
        {"name": f"lever_additional_tag_{i}", "value": tag}
        for i, tag in enumerate(additional_tags)
    )

    distribution_channels = lever_job.get("distributionChannels", [])
    tags.extend(
        {"name": f"lever_distributionChannel_{index}", "value": channel}
        for index, channel in enumerate(distribution_channels)
    )

    urls = lever_job.get("urls", {})
    tags.extend(
        {"name": f"lever_url_{item}", "value": value} for item, value in urls.items()
    )

    country = lever_job.get("country")
    if country:
        tags.append({"name": "lever_country", "value": country})

    state = lever_job.get("state")
    if state:
        tags.append({"name": "lever_state", "value": state})
    workplaceType = lever_job.get("workplaceType")
    if workplaceType:
        tags.append({"name": "lever_workplaceType", "value": workplaceType})

    return tags


# get ranges float
def get_job_ranges_float(lever_job: dict) -> list:
    ranges_list = []
    ranges_float = {}
    salary_range = lever_job.get("salaryRange", {})
    if salary_range:
        ranges_float["name"] = "salary"
        ranges_float["value_min"] = salary_range.get("min", None)
        ranges_float["value_max"] = salary_range.get("max", None)
        ranges_float["unit"] = salary_range.get("currency", None)
        ranges_list.append(ranges_float)
    return ranges_list


def format_job(lever_job: dict) -> dict:
    lever_job_data = lever_job

    created_at = datetime.fromtimestamp(
        int(lever_job_data["createdAt"]) / 1000
    ).isoformat()
    updated_at = datetime.fromtimestamp(
        int(lever_job_data["updatedAt"]) / 1000
    ).isoformat()

    job = dict(
        reference=lever_job_data["id"],
        name=lever_job_data["text"],
        location=dict(
            text=(
                lever_job_data.get("categories", {}).get("location", None)
                if lever_job_data.get("categories", {}).get("location", None)
                else None
            ),
            lat=None,
            lng=None,
        ),
        url=lever_job_data["urls"]["show"],
        summary=lever_job_data["content"]["description"],
        sections=extract_job_sections(lever_job_data["content"]),
        tags=get_job_tags(lever_job_data),
        ranges_float=get_job_ranges_float(lever_job_data),
        created_at=created_at,
        updated_at=updated_at,
    )
    return job


def get_profile_experiences(lever_profile_data: t.Dict[str, any]) -> t.List:
    if not lever_profile_data:
        return []
    experiences_data = lever_profile_data.get("parsedData", {}).get("positions", [])
    experiences = []

    for experience_data in experiences_data:
        company = experience_data.get("org")
        title = experience_data.get("title")
        description = experience_data.get("summary")
        location = dict(
            text=experience_data.get("location"),
            lat=None,
            lng=None,
        )
        start_data = experience_data.get("start", {})
        end_data = experience_data.get("end", {})
        start_year = start_data.get("year")
        start_month = start_data.get("month")
        end_year = end_data.get("year")
        end_month = end_data.get("month")

        start_date = (
            datetime(start_year, start_month, 1).isoformat()
            if start_year and start_month
            else None
        )
        end_date = (
            datetime(end_year, end_month, 1).isoformat()
            if end_year and end_month
            else None
        )

        experience = dict(
            company=company,
            title=title,
            description=description,
            location=location,
            date_start=start_date,
            date_end=end_date,
            skills=[],
        )
        experiences.append(experience)

    return experiences


def get_profile_educations(lever_profile_data: t.Dict[str, any]) -> t.List:
    if not lever_profile_data:
        return []
    educations_data = lever_profile_data.get("parsedData", {}).get("schools", [])
    educations = []

    for education_data in educations_data:
        school = education_data.get("org")
        title = education_data.get("degree")
        description = education_data.get("summary")
        location = dict(
            text=education_data.get("location"),
            lat=None,
            lng=None,
        )
        start_data = education_data.get("start", {})
        end_data = education_data.get("end", {})
        start_year = start_data.get("year")
        start_month = start_data.get("month")
        end_year = end_data.get("year")
        end_month = end_data.get("month")
        start_date = (
            datetime(start_year, start_month, 1).isoformat()
            if start_year and start_month
            else None
        )
        end_date = (
            datetime(end_year, end_month, 1).isoformat()
            if end_year and end_month
            else None
        )

        education = dict(
            school=school,
            title=title,
            description=description,
            location=location,
            date_start=start_date,
            date_end=end_date,
            skills=[],
        )
        educations.append(education)

    return educations


def get_profile_attachments(lever_profile_data: t.Dict[str, any]) -> t.List[str]:
    attachments = []
    if not lever_profile_data:
        return []
    attachments_data = lever_profile_data.get("file", {})
    if attachments_data:
        attachment_dict = dict()
        attachment_dict["type"] = "original"
        attachment_dict["file_name"] = attachments_data["name"]
        attachment_dict["original_file_name"] = attachments_data["name"]
        attachment_dict["extension"] = attachments_data["ext"]
        attachment_dict["public_url"] = attachments_data["downloadUrl"]
        attachment_dict["file_size"] = attachments_data["size"]
        attachment_dict["created_at"] = datetime.fromtimestamp(
            int(attachments_data["uploadedAt"]) / 1000
        ).isoformat()
        attachments.append(attachment_dict)
    return attachments


def format_profile_urls(lever_links: dict) -> dict:
    urls = []
    if not lever_links:
        return []
    for link in lever_links:
        url = {"type": "from lever", "url": link}
        urls.append(url)
    return urls


def extract_first_last_name(full_name):
    names = full_name.split()
    if len(names) > 1:
        first_name = names[0]
        last_name = " ".join(names[1:])
    else:
        first_name = full_name
        last_name = None
    return first_name, last_name


def format_profile(opportunity_data: dict) -> dict:
    lever_opportunity = opportunity_data
    lever_profile = (
        lever_opportunity.get("profile")[0] if lever_opportunity.get("profile") else {}
    )
    created_at = datetime.fromtimestamp(
        int(lever_opportunity["createdAt"]) / 1000
    ).isoformat()
    updated_at = datetime.fromtimestamp(
        int(lever_opportunity["updatedAt"]) / 1000
    ).isoformat()
    first_name, last_name = extract_first_last_name(lever_opportunity.get("name"))
    profile = dict(
        reference=lever_opportunity["id"],
        updated_at=updated_at,
        created_at=created_at,
        info={
            "full_name": lever_opportunity.get("name"),
            "first_name": first_name,
            "last_name": last_name,
            "email": (
                lever_opportunity.get("emails")[0]
                if lever_opportunity.get("emails")
                else None
            ),
            "phone": (
                lever_opportunity.get("phones")[0].get("value")
                if lever_opportunity.get("phones")
                else None
            ),
            "location": (
                dict(text=lever_opportunity.get("location"), lat=None, lng=None)
                if lever_opportunity.get("location")
                else dict(text=None, lat=None, lng=None)
            ),
            "urls": format_profile_urls(lever_opportunity.get("links")),
        },
        text_language=None,
        text=(
            lever_opportunity.get("headline")
            if lever_opportunity.get("headline")
            else None
        ),
        experiences=get_profile_experiences(lever_profile),
        educations=get_profile_educations(lever_profile),
        attachments=get_profile_attachments(lever_profile),
        skills=[],
        tags=[],
    )
    return profile


# get profile urls
def get_profile_urls(urls: list) -> list:
    urls_list = []
    if not urls:
        return []
    for url_dict in urls:
        urls_list.append(url_dict["url"])
    return urls_list


# get profile skills
def get_profile_skills(skills: list) -> list:
    lever_tags = []
    if not skills:
        return []
    for skill_dict in skills:
        lever_tags.append(skill_dict["name"])
    return lever_tags


# transform date from iso to timestamp
def from_iso_to_timestamp(date: str) -> str:
    if not date:
        return None
    try:
        dt = datetime.strptime(date, "%Y-%m-%dT%H:%M:%S.%fZ")
    except ValueError:
        dt = datetime.strptime(date, "%Y-%m-%dT%H:%M:%S%z")

    timestamp = int(dt.timestamp() * 1000)  # Convert to milliseconds
    return timestamp


# construct headline
def consruct_headline(hrflow_profile: dict) -> str:
    experiences = hrflow_profile["experiences"]
    educations = hrflow_profile["educations"]
    list_of_entity_title = []
    if experiences:
        for experience in experiences:
            list_of_entity_title.append(experience["company"])
    if educations:
        for education in educations:
            list_of_entity_title.append(education["school"])
    if not list_of_entity_title:
        return None
    return " , ".join(list_of_entity_title)


def format_opportunity(hrflow_profile: dict) -> dict:
    lever_opportunity = dict()
    lever_opportunity["name"] = hrflow_profile["info"]["full_name"]
    lever_opportunity["headline"] = consruct_headline(hrflow_profile)
    lever_opportunity["location"] = hrflow_profile["info"]["location"]["text"]
    lever_opportunity["phones"] = [
        {"type": "mobile", "value": hrflow_profile["info"]["phone"]}
    ]
    lever_opportunity["emails"] = [hrflow_profile["info"]["email"]]
    lever_opportunity["links"] = get_profile_urls(hrflow_profile["info"]["urls"])
    lever_opportunity["tags"] = get_profile_skills(hrflow_profile["skills"])
    lever_opportunity["createdAt"] = from_iso_to_timestamp(hrflow_profile["created_at"])
    if hrflow_profile["attachments"]:
        lever_opportunity["file"] = hrflow_profile["attachments"][0]
    return lever_opportunity


DESCRIPTION = (
    "Lever is a modern recruitment platform that helps companies streamline their"
    " hiring process."
)
Lever = Connector(
    name="Lever",
    type=ConnectorType.ATS,
    description=DESCRIPTION,
    url="https://www.lever.co/",
    actions=[
        ConnectorAction(
            name=ActionName.pull_job_list,
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the Lever API and sends them to the Hrflow.ai"
                " Board."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=LeverJobWarehouse,
            target=HrFlowJobWarehouse,
            action_type=ActionType.inbound,
        ),
        ConnectorAction(
            name=ActionName.push_profile,
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from the Hrflow.ai Source to Lever via the API."
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_opportunity
            ),
            origin=HrFlowProfileWarehouse,
            target=LeverProfileWarehouse,
            action_type=ActionType.outbound,
        ),
        ConnectorAction(
            name=ActionName.pull_profile_list,
            trigger_type=WorkflowType.pull,
            description="Read a profile from Lever Source to Hrflow.ai via the API.",
            parameters=BaseActionParameters.with_defaults(
                "ReadProfilesParameters", format=format_profile
            ),
            origin=LeverProfileWarehouse,
            target=HrFlowProfileWarehouse,
            action_type=ActionType.inbound,
        ),
    ],
)



import base64
import typing as t
from logging import LoggerAdapter

import magic
from pydantic import Field

from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
)


class ReadProfilesParameters(ParametersModel):
    profile: t.Dict = Field(
        None,
        description="Event object recieved from the Webhook",
        field_type=FieldType.Other,
    )


def get_content_type(binary_data: bytes):
    mime = magic.Magic(mime=True)
    content_type = mime.from_buffer(binary_data)
    if not content_type:
        return "application/octet-stream"
    return content_type


def read(
    adapter: LoggerAdapter,
    parameters: ReadProfilesParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    result = {**parameters.profile}
    cv_base64 = result.get("cvBase64")

    if cv_base64 is None:
        raise ValueError("No base64 string provided for CV.")

    try:
        binary_data = base64.b64decode(cv_base64)
    except base64.binascii.Error:
        padding_needed = 4 - (len(cv_base64) % 4)
        if padding_needed != 4:
            cv_base64 += "=" * padding_needed
            binary_data = base64.b64decode(cv_base64)

    if not binary_data:
        raise Exception("Error decoding base64 string")
    content_type = get_content_type(binary_data)
    result["cv"] = binary_data
    result["content_type"] = content_type

    return [result]


JobologyProfilesWarehouse = Warehouse(
    name="Jobology Candidate",
    data_type=DataType.profile,
    read=WarehouseReadAction(
        parameters=ReadProfilesParameters,
        function=read,
    ),
)



import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.workable.schemas import WorkableJobModel
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
)
from hrflow_connectors.core.warehouse import WarehouseWriteAction


class WorkableReadParameters(ParametersModel):
    auth: str = Field(..., description="API KEY", field_type=FieldType.Auth)
    subdomain: str = Field(..., description="Subdomain", field_type=FieldType.Auth)


class WorkableWriteParameters(ParametersModel):
    auth: str = Field(..., description="API KEY", field_type=FieldType.Auth)
    subdomain: str = Field(..., description="Subdomain", field_type=FieldType.Other)
    shortcode: str = Field(..., description="Job shortcode", field_type=FieldType.Other)


def read(
    adapter: LoggerAdapter,
    parameters: WorkableReadParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterator[WorkableJobModel]:
    url = f"https://{parameters.subdomain}.workable.com/spi/v3/jobs?state=published"

    payload = {}
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {parameters.auth}",
    }

    response = requests.get(url, headers=headers, data=payload)

    if not response.ok:
        adapter.error(f"Fail to read Workable jobs reason : {response.content}")

    for job in response.json().get("jobs"):
        yield job


def write(
    adapter: LoggerAdapter,
    parameters: WorkableWriteParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    failed_profiles = []

    for profile in profiles:
        url = (
            f"https://{parameters.subdomain}.workable.com/"
            f"spi/v3/jobs/{parameters.shortcode}/candidates"
        )

        payload = profile

        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {parameters.auth}",
            "Accept": "application/json",
        }

        response = requests.post(url, headers=headers, data=payload)
        if not response.ok:
            adapter.error(f"Error append with this profile : {profile}")
            failed_profiles.append(profile)

    return failed_profiles


WorkableJobWarehouse = Warehouse(
    name="WorkableJobWarehouse",
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=WorkableReadParameters,
        function=read,
    ),
)

WorkableProfileWarehouse = Warehouse(
    name="WorkableProfileWarehouse",
    data_type=DataType.job,
    write=WarehouseWriteAction(
        parameters=WorkableWriteParameters,
        function=write,
    ),
)



from typing import Any, Dict, List, Optional

from pydantic import BaseModel


# Job model
class SAPSuccessFactorsJobRequistion(BaseModel):
    annual_SA: Optional[str]
    location: Optional[str]
    city: Optional[str] = None
    country: Optional[str] = None
    department: Optional[str] = None
    division: Optional[str] = None
    facility: Optional[str] = None
    function: Optional[str] = None
    industry: Optional[str] = None
    monthly_salary: Optional[str] = None
    salaryBase: Optional[str] = None
    otherBonus: Optional[str] = None
    salaryMax: Optional[str] = None
    salaryMin: Optional[str] = None
    stateProvince: Optional[str] = None
    jobStartDate: Optional[str] = None
    recruiterTeam: Optional[Dict[str, Any]] = None
    hiringManagerTeam: Optional[Dict[str, Any]] = None
    sourcerTeam: Optional[Dict[str, Any]] = None


class SAPSuccessFactorsJob(BaseModel):
    jobDescription: Optional[str]
    jobTitle: Optional[str]
    jobReqId: Optional[str]
    jobRequisition: SAPSuccessFactorsJobRequistion


# Profile model


class Result(BaseModel):
    endDate: Optional[str]
    school: str
    schoolAddress: str
    startDate: Optional[str]


class Education(BaseModel):
    results: List[Result]


class ResultLanguage(BaseModel):
    language: str
    readingProf: str
    speakingProf: str
    writingProf: str


class ResultOutsideWorkExperience(BaseModel):
    employer: Optional[str]
    employerAddress: str
    endDate: Optional[str]
    startDate: Optional[str]


class OutsideWorkExperience(BaseModel):
    results: List[ResultOutsideWorkExperience]


class InsideWorkExperienceResult(BaseModel):
    backgroundElementId: Optional[str]
    bgOrderPos: Optional[str]
    candidateId: Optional[str]
    department: Optional[str]
    endDate: Optional[str]
    lastModifiedDateTime: Optional[str]
    startDate: Optional[str]
    title: Optional[str]
    candidate: Optional[str]


class TalentPoolResults(BaseModel):
    startDate: Optional[str]
    talentPoolComments: Optional[str]
    talentPoolStatus: Optional[str]
    talentPoolitem: Optional[str]


class TalentPool(BaseModel):
    results: List[TalentPoolResults]


class SapCandidateModel(BaseModel):
    address: Optional[str]
    cellPhone: Optional[str]
    city: Optional[str]
    contactEmail: Optional[str]
    country: Optional[str]
    currentTitle: Optional[str]
    firstName: Optional[str]
    homePhone: Optional[str]
    lastName: Optional[str]
    middleName: Optional[str]
    primaryEmail: str
    zip: Optional[str]
    education: Optional[Education]
    outsideWorkExperience: Optional[OutsideWorkExperience]



from typing import Any, Optional

from pydantic import BaseModel, Field


class BullhornAddress(BaseModel):
    address1: Optional[str] = Field(None, Description="Adress of the profile")
    city: Optional[str] = Field(None, Description="City of the profile")
    state: Optional[str] = Field(None, Description="Country code of the profile")
    zip: Optional[str] = Field(None, Description="Postal code of the profile")


class BullhornProfile(BaseModel):
    id: Optional[str] = Field(None, Description="Unique identifier for this entity")
    address: Optional[BullhornAddress] = Field(None, Description="Candidate address")
    certifications: Any = Field(None, Description="Candidate’s certifications")
    name: Optional[str] = Field(
        None,
        Description=(
            "Candidate’s full name. If setting firstname or lastname, you must also set"
            " this field; it does not populate automatically"
        ),
    )
    firstName: Optional[str] = Field(None, Description="Candidate’s first name")
    lastName: Optional[str] = Field(None, Description="Name of the file")
    email: Optional[Optional[str]] = Field(
        None, Description="Candidate’s email address"
    )
    mobile: Optional[Optional[str]] = Field(
        None, Description="Candidate’s mobile (cell) telephone number"
    )
    dateOfBirth: Optional[int] = Field(None, Description="Candidate’s date of birth")
    experience: Optional[int] = Field(
        None, Description="Number of years of experience that the Candidate has"
    )
    skillSet: Optional[str] = Field(
        None, Description="Text description of Candidate’s skills"
    )


class BullhornAttachmentEnrichment(BaseModel):
    externalID: Optional[str] = Field(
        None, Description="External identifier for the file"
    )
    fileContent: Optional[str] = Field(
        None, Description="Base64-encoded Optional[str]ing of the file content"
    )
    fileExtension: Optional[Optional[str]] = Field(
        None, Description="Extension of the file. For example, .doc or .jpg"
    )
    fileType: Optional[str] = Field(None, Description="Always use the value “SAMPLE”")
    name: Optional[str] = Field(
        None,
        Description=(
            "File name. If a file extension is included as part of the name and the "
            "fileExtension field is not set, the file extension in the name is used."
        ),
    )
    contentType: Optional[str] = Field(
        None, Description="Type/subtype of the file content.type"
    )
    description: Optional[str] = Field(
        None, Description="Unique identifier for this entity"
    )
    type: Optional[str]


class BullhornCandidate(BaseModel):
    id: Optional[Optional[int]] = Field(
        None, Description="Unique identifier for this entity"
    )


class BullhornExperienceEnrichment(BaseModel):
    id: Optional[str] = Field(None, Description="Unique identifier for this entity")
    candidate: BullhornCandidate = Field(
        None, Description="Candidate for whom this person is a reference"
    )
    companyName: Optional[Optional[str]] = Field(
        None,
        Description=(
            "Name of the company where reference works, if it does not "
            "have a ClientCorporation record in Bullhorn"
        ),
    )
    title: Optional[Optional[str]] = Field(
        None, Description="Candidate’s job title in this position"
    )
    comments: Optional[Optional[str]] = Field(
        None, Description="Free-text comments on CandidateWorkHistory"
    )
    startDate: Optional[Optional[int]] = Field(
        None, Description="Date on which Candidate began working at this position"
    )
    endDate: Optional[Optional[int]] = Field(
        None, Description="Date on which job ended, if applicable"
    )


class BullhornEducationEnrichment(BaseModel):
    id: Optional[str] = Field(None, Description="Unique identifier for this entity")
    candidate: BullhornCandidate = Field(
        None, Description="Candidate for whom this person is a reference"
    )
    school: Optional[Optional[str]] = Field(
        None,
        Description="Name of the educational institute where this education took place",
    )
    degree: Optional[Optional[str]] = Field(
        None,
        Description=(
            "Indicates what educational degree the Candidate received; for "
            "example, B.A., M.A., Ph.D., and so forth"
        ),
    )
    comments: Optional[Optional[str]] = Field(
        None, Description="Free-text comments on this record"
    )
    city: Optional[Optional[str]] = Field(
        None, Description="Name of the city where the education took place"
    )
    startDate: Optional[Optional[int]] = Field(
        None, Description="Date when Candidate began study"
    )
    endDate: Optional[Optional[int]] = Field(
        None, Description="Date when Candidate finished this education"
    )


class BullhornJob(BaseModel):
    id: Optional[Optional[int]] = Field(
        None, Description="Unique identifier for this entity"
    )



import typing as t

from pydantic import BaseModel


class LeverJobContent(BaseModel):
    description: str
    descriptionHtml: str
    lists: t.List[t.Dict[str, str]]
    closing: str
    closingHtml: str


class LeverJobCategories(BaseModel):
    commitment: str
    department: str
    level: str
    location: str
    team: str


class LeverJobUrls(BaseModel):
    list: str
    show: str
    apply: str


class LeverJobSalaryRange(BaseModel):
    min: int
    max: int
    currency: str
    interval: str


class LeverJob(BaseModel):
    id: str
    text: str
    state: str
    distributionChannels: t.List[str]
    user: str
    owner: str
    hiringManager: str
    categories: LeverJobCategories
    tags: t.List[str]
    content: LeverJobContent
    country: str
    followers: t.List[str]
    reqCode: str
    requisitionCodes: t.List[str]
    urls: LeverJobUrls
    confidentiality: str
    createdAt: int
    updatedAt: int
    workplaceType: str
    salaryRange: LeverJobSalaryRange


class LeverProfile(BaseModel):
    id: str
    name: t.Dict[str, str]
    email: t.Optional[str]
    phone: t.Optional[str]
    createdAt: str
    updatedAt: str



import re
import typing as t
from datetime import datetime

from hrflow_connectors.connectors.digitalrecruiters.warehouse import (
    DigitalRecruitersJobWarehouse,
    DigitalRecruitersReadProfilesWarehouse,
    DigitalRecruitersWriteProfileWarehouse,
)
from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileParsingWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.core import (
    ActionName,
    ActionType,
    BaseActionParameters,
    Connector,
    ConnectorAction,
    ConnectorType,
    WorkflowType,
)


def html_to_plain_text(html_text):
    if html_text is None:
        return None
    # Remove HTML tags
    plain_text = re.sub(r"<.*?>", "", html_text)

    # Replace special characters with their plain text equivalents
    plain_text = plain_text.replace("&nbsp;", " ")
    plain_text = plain_text.replace("&amp;", "&")
    plain_text = plain_text.replace("&quot;", '"')
    plain_text = plain_text.replace("&apos;", "'")
    plain_text = plain_text.replace("&lt;", "<")
    plain_text = plain_text.replace("&gt;", ">")

    # Remove extra whitespace and newline characters
    plain_text = re.sub(r"\s+", " ", plain_text).strip()

    return plain_text


def get_job_location(digital_recruiters_adress: t.Union[t.Dict, None]) -> t.Dict:
    if not digital_recruiters_adress:
        return dict(lat=None, lng=None, text="")

    lat = digital_recruiters_adress.get("position", {}).get("lat", None)
    lat = float(lat) if lat is not None else lat

    lng = digital_recruiters_adress.get("position", {}).get("lon", None)
    lng = float(lng) if lng is not None else lng
    text = digital_recruiters_adress.get("formatted", None)

    return dict(lat=lat, lng=lng, text=text)


def get_sections(digital_recruiters_job: t.Dict) -> t.List[t.Dict]:
    sections = []
    if (
        "description" not in digital_recruiters_job
        or "profile" not in digital_recruiters_job
    ):
        return sections

    for section_name in [
        "description",
        "profile",
    ]:
        section = digital_recruiters_job.get(section_name, None)
        if section is not None:
            sections.append(
                dict(
                    name=section_name,
                    title=section_name,
                    description=html_to_plain_text(section),
                )
            )
    return sections


def get_tags(digital_recruiters_job: t.Dict) -> t.List[t.Dict]:
    job = digital_recruiters_job
    custom_field_mapping = {
        "Possibilité de télétravail": "digitalrecruiters_possibilite_de_teletravail",
        "Automatisation (HRFlow.ai)": "digitalrecruiters_automatisation_hrflow",
        "Heures hebdomadaires": "digitalrecruiters_heures_hebdomadaires",
        "Date envisagée de recrutement": (
            "digitalrecruiters_date_enviseagee_de_recrutement"
        ),
        "Date de fin": "digitalrecruiters_date_de_fin",
        "Motif de recrutement": "digitalrecruiters_motif_de_recrutement",
        "Nom de la personne remplacée": (
            "digitalrecruiters_nom_de_la_personne_remplacee"
        ),
        "Echelon": "digitalrecruiters_echelon",
        "Filière": "digitalrecruiters_filiere",
        "Horaires": "digitalrecruiters_horaires",
        "Un candidat est déjà identifié": "digitalrecruiters_candidat_deja_identifie",
        "Nom de ce candidat": "digitalrecruiters_nom_du_candidat",
    }

    tags = []

    def add_tag(name, value):
        if value is not None and value != "":
            tags.append({"name": name, "value": value})

    compensation = job.get("salary", {})
    if compensation:
        add_tag("digitalrecruiters_compensation_min", compensation.get("min", None))
        add_tag("digitalrecruiters_compensation_max", compensation.get("max", None))
        add_tag(
            "digitalrecruiters_compensation_currency",
            compensation.get("currency", None),
        )

    manager = job.get("entity", {}).get("manager", {})
    if manager:
        add_tag("digitalrecruiters_manager_firstName", manager.get("firstname", None))
        add_tag("digitalrecruiters_manager_lastName", manager.get("lastname", None))
        add_tag("digitalrecruiters_manager_position", manager.get("position", None))
        add_tag("digitalrecruiters_manager_picture", manager.get("picture_url", None))

    recruiter = job.get("referent_recruiter", {})
    if recruiter:
        add_tag("digitalrecruiters_recruiter_email", recruiter.get("email", None))
        add_tag(
            "digitalrecruiters_recruiter_phoneNumber",
            recruiter.get("phoneNumber", None),
        )
        add_tag(
            "digitalrecruiters_recruiter_picture", recruiter.get("picture_url", None)
        )

    hierarchy_list = digital_recruiters_job.get("hierarchy", [])
    for item in hierarchy_list:
        depth = item.get("depth", None)
        column_name = item.get("column_name", None)
        public_name = item.get("public_name", None)
        add_tag(f"hierarchy_{depth}", f"{column_name}:{public_name}")

    custom_fields = job.get("custom_fields", [])
    if custom_fields:
        for custom_field in custom_fields:
            name = custom_field.get("name", None)
            value = custom_field.get("value", None)
            mapped_name = custom_field_mapping.get(name, None)
            if mapped_name:
                add_tag(mapped_name, value)

    return tags


def format_skills(skills_list):
    formatted_skills = [
        {"name": skill, "type": None, "value": None} for skill in skills_list
    ]
    return formatted_skills


# format profile location retrieved from DigitlRecruiters
def get_profile_location(Dr_location: t.Dict) -> t.Dict:
    if not Dr_location:
        return dict(text="", lat=None, lng=None)
    street = Dr_location.get("street", "")
    zip_code = Dr_location.get("zip", "")
    city = Dr_location.get("city", "")
    country = Dr_location.get("country", "")

    parts = []

    if street:
        parts.append(street)

    if city:
        parts.append(city)

    if zip_code:
        parts.append(f"({zip_code})")

    if country:
        parts.append(country)

    location_text = ", ".join(parts)
    location_lat = Dr_location.get("latitude", None)
    location_lng = Dr_location.get("longitude", None)
    return dict(text=location_text, lat=location_lat, lng=location_lng)


def normalize_link(link):
    normalized_link = link.replace("\\", "")
    return normalized_link


def format_job(digital_recruiters_job: t.Dict) -> t.Dict:
    picture = None
    pictures = digital_recruiters_job.get("pictures", [])
    if pictures:
        picture = pictures[0].get("default", None)
    job = dict(
        name=digital_recruiters_job.get("title", None),
        picture=picture,
        reference=digital_recruiters_job.get("reference", None),
        created_at=digital_recruiters_job.get("published_at", None),
        location=get_job_location(digital_recruiters_job.get("address", {})),
        sections=get_sections(digital_recruiters_job),
        requirements=html_to_plain_text(digital_recruiters_job.get("profile", None)),
        skills=format_skills(digital_recruiters_job.get("skills", [])),
        tags=get_tags(digital_recruiters_job),
    )
    return job


def format_dr_profile(dr_candidate: t.Dict) -> t.Dict:
    full_name = (
        f"{dr_candidate.get('firstName', None)} {dr_candidate.get('lastName', None)}"
    )
    resume = dr_candidate.get("cv", {}).get("url", None)
    resume = normalize_link(resume)
    avatar = dr_candidate.get("avatar", {}).get("url", None)
    reference = dr_candidate.get("id")
    created_at = dr_candidate.get("createdAt")
    resume = dr_candidate.get("resume")
    location = get_profile_location(dr_candidate.get("location", {}))
    # add tags
    tags = []

    def add_tag(name, value):
        if value is not None:
            tags.append({"name": name, "value": value})

    add_tag("digitalrecruiters_profile-email", dr_candidate.get("email", None))
    add_tag(
        "digitalrecruiters_profile-phoneNumber", dr_candidate.get("phoneNumber", None)
    )
    add_tag("digitalrecruiters_profile-fullName", full_name)
    add_tag("digitalrecruiters_avatar", avatar)
    add_tag("digitalrecruiters_profile-resume", resume)
    add_tag("digitalrecruiters_profile-location", location.get("text", None))
    add_tag(
        "digitalrecruiters_education-level", dr_candidate.get("educationLevel", None)
    )
    add_tag(
        "digitalrecruiters_job-experience-level",
        dr_candidate.get("experienceLevel", None),
    )
    add_tag("digitalrecruiters_job-title", dr_candidate.get("jobTitle", None))
    add_tag("digitalrecruiters_job-id", dr_candidate.get("jobAd", {}).get("id", None))
    add_tag(
        "digitalrecruiters_job-published-at",
        dr_candidate.get("jobAd", {}).get("publishedAt", None),
    )
    add_tag("digitalrecruiters_locale", dr_candidate.get("locale", None))
    add_tag("digitalrecruiters_origin", dr_candidate.get("origin", None))
    add_tag("digitalrecruiters_is-spontaneous", dr_candidate.get("isSpontaneous", None))
    add_tag("digitalrecruiters_is-imported", dr_candidate.get("isImported", None))
    add_tag(
        "digitalrecruiters_is-from-external-api",
        dr_candidate.get("isFromExternalApi", None),
    )
    add_tag(
        "digitalrecruiters_rejected-reason", dr_candidate.get("rejectedReason", None)
    )
    add_tag(
        "digitalrecruiters_application-status",
        dr_candidate.get("applicationStatus", None),
    )

    metadatas = []
    profile_hrflow = dict(
        reference=reference,
        created_at=created_at,
        updated_at=datetime.utcnow().isoformat(),
        resume=resume,
        tags=tags,
        metadatas=metadatas,
    )
    return profile_hrflow


def format_profile(profile_hrflow: t.Dict) -> t.Dict:
    dr_profile_dict = dict()

    # Gender mapping: 1 for Male, 2 for Female
    dr_profile_dict["gender"] = (
        1 if profile_hrflow["info"]["gender"].lower() == "male" else 2
    )
    dr_profile_dict["firstName"] = profile_hrflow["info"]["first_name"]
    dr_profile_dict["lastName"] = profile_hrflow["info"]["last_name"]
    dr_profile_dict["email"] = profile_hrflow["info"]["email"]
    dr_profile_dict["phoneNumber"] = profile_hrflow["info"].get("phone")
    location = profile_hrflow["info"].get("location", {})
    dr_profile_dict["addressStreet"] = location.get("text")

    fields = location.get("fields", {})
    if isinstance(fields, list):
        # If fields is a list, take the first element as it's a dictionary
        fields = fields[0]
    dr_profile_dict["addressZip"] = fields.get("postcode")
    dr_profile_dict["addressCity"] = fields.get("state_district")

    profile = dict()
    profile["consent_date"] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    profile["s_o"] = profile_hrflow.get("s_o", "")
    profile["locale"] = "fr_FR"
    profile["ApplicationProfile"] = dr_profile_dict

    return profile


DESCRIPTION = (
    "Digital Recruiters: Tech-driven hiring platform with job posting,"
    " automation, and analytics. Simplify recruitment, reduce time-to-hire,"
    " and elevate candidate experience. Streamline the hiring process for"
    " businesses with advanced features and integration capabilities."
)

DigitalRecruiters = Connector(
    name="DigitalRecruiters",
    type=ConnectorType.ATS,
    description=DESCRIPTION,
    url="https://www.digitalrecruiters.com/",
    actions=[
        ConnectorAction(
            name=ActionName.pull_job_list,
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs from Digital Recruiters and sends them to an"
                " Hrflow.ai Board."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=DigitalRecruitersJobWarehouse,
            target=HrFlowJobWarehouse,
            action_type=ActionType.inbound,
        ),
        ConnectorAction(
            name=ActionName.pull_profile_list,
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all profiles from Digital Recruiters and sends them to an"
                " Hrflow.ai Source."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadProfilesActionParameters", format=format_dr_profile
            ),
            origin=DigitalRecruitersReadProfilesWarehouse,
            target=HrFlowProfileParsingWarehouse,
            action_type=ActionType.inbound,
        ),
        ConnectorAction(
            name=ActionName.push_profile,
            trigger_type=WorkflowType.catch,
            description="Pushes a profile from Hrflow.ai to Digital Recruiters.",
            parameters=BaseActionParameters.with_defaults(
                "WriteProfilesActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=DigitalRecruitersWriteProfileWarehouse,
            action_type=ActionType.outbound,
        ),
    ],
)



import enum
import typing as t
from logging import LoggerAdapter

from pydantic import BaseModel, Field, ValidationError, create_model, root_validator
from pydantic.fields import FieldInfo
from pydantic.main import ModelMetaclass


class FieldNotFoundError(RuntimeError):
    pass


class FixedValueValidationError(RuntimeError):
    pass


class InvalidFieldError(TypeError):
    pass


class NoFieldTypeError(TypeError):
    pass


class BadFieldTypeError(TypeError):
    pass


class DataType(enum.Enum):
    profile = enum.auto()
    job = enum.auto()
    other = enum.auto()


class ActionType(enum.Enum):
    read = enum.auto()
    write = enum.auto()


class ReadMode(enum.Enum):
    sync = "sync"
    incremental = "incremental"


class FieldType(str, enum.Enum):
    Auth = "Auth"
    QueryParam = "Query Param"
    Other = "Other"


class ActionEndpoints(BaseModel):
    name: str
    description: str
    url: str


FIELD_TYPE_EXAMPLE = """
    Example :
        from pydantic import Field

        from hrflow_connectors.core import FieldType

        class MyParams(ParametersModel):
            my_field: str = Field(
                ..., description="My field", field_type=FieldType.Other
            )
"""
INVALID_FIELD_ERROR_MSG = """Field '{{}}' in {{}} should have proper annotation using pydantic.Field.
    {}
""".format(
    FIELD_TYPE_EXAMPLE
)
NO_FIELD_TYPE_ERROR_MSG = """Field '{{}}' in {{}} is missing 'field_type' declaration.
    {}
""".format(FIELD_TYPE_EXAMPLE)
BAD_FIELD_TYPE_ERROR_MSG = """'field_type' for field '{{}}' in {{}} should be defined using
    `hrflow_connectors.core.FieldType`.
    {}
""".format(
    FIELD_TYPE_EXAMPLE
)


class ParametersMeta(ModelMetaclass):
    def __new__(self, name, bases, namespaces, **kwargs):
        for annotation in namespaces.get("__annotations__", {}).keys():
            field_info = namespaces.get(annotation)
            if field_info is None or not isinstance(field_info, FieldInfo):
                raise InvalidFieldError(
                    INVALID_FIELD_ERROR_MSG.format(annotation, name)
                )
            field_type = field_info.extra.get("field_type")
            if field_type is None:
                raise NoFieldTypeError(NO_FIELD_TYPE_ERROR_MSG.format(annotation, name))
            if not isinstance(field_type, FieldType):
                raise BadFieldTypeError(
                    BAD_FIELD_TYPE_ERROR_MSG.format(annotation, name)
                )

        return super().__new__(self, name, bases, namespaces, **kwargs)


class ParametersModel(BaseModel, metaclass=ParametersMeta):
    class Config:
        extra = "forbid"


class WarehouseReadAction(BaseModel):
    endpoints: t.List[ActionEndpoints] = Field(default_factory=list)
    parameters: t.Type[ParametersModel]
    function: t.Callable[
        [LoggerAdapter, ParametersModel, t.Optional[ReadMode], t.Optional[str]],
        t.Iterable[t.Dict],
    ]
    item_to_read_from: t.Optional[t.Callable[[t.Dict], str]] = None
    supports_incremental: bool = False

    def __call__(self, *args, **kwargs) -> t.Iterable[t.Dict]:
        return self.function(*args, **kwargs)

    @root_validator
    def validate_incremental(cls, values):
        supports_incremental = values.get("supports_incremental")
        item_to_read_from = values.get("item_to_read_from")
        if supports_incremental is True and item_to_read_from is None:
            raise ValueError(
                "Function item_to_read_from must be provided when"
                " supports_incremental is True"
            )
        return values


class WarehouseWriteAction(BaseModel):
    endpoints: t.List[ActionEndpoints] = Field(default_factory=list)
    parameters: t.Type[ParametersModel]
    function: t.Callable[
        [LoggerAdapter, ParametersModel, t.Iterable[t.Dict]], t.List[t.Dict]
    ]

    def __call__(self, *args, **kwargs) -> t.List[t.Dict]:
        return self.function(*args, **kwargs)


class Warehouse(BaseModel):
    name: str
    data_type: DataType
    data_schema: t.Type[BaseModel] = Field(default_factory=lambda: BaseModel)
    read: t.Optional[WarehouseReadAction]
    write: t.Optional[WarehouseWriteAction]

    @property
    def supports_incremental(self):
        return self.read.supports_incremental

    def item_to_read_from(self, *args, **kwargs):
        return self.read.item_to_read_from(*args, **kwargs)

    @property
    def is_readable(self):
        return self.read is not None

    @property
    def is_writable(self):
        return self.write is not None

    def with_fixed_read_parameters(self, **tofix) -> "Warehouse":
        return self.__with_fixed_parameters(action_type=ActionType.read, **tofix)

    def with_fixed_write_parameters(self, **tofix) -> "Warehouse":
        return self.__with_fixed_parameters(action_type=ActionType.write, **tofix)

    def __with_fixed_parameters(self, action_type: ActionType, **tofix) -> "Warehouse":
        action_to_fix = getattr(self, action_type.name)
        fixed = dict()
        original_fields = action_to_fix.parameters.__fields__
        for field, value in tofix.items():
            if field not in original_fields:
                raise FieldNotFoundError(
                    "The field you are trying to fix '{}' is not part of the available"
                    " parameters {}".format(field, list(original_fields.keys()))
                )
            try:
                action_to_fix.parameters(**{field: value})
            except ValidationError as e:
                errors = e.errors()
                field_error = next(
                    (error for error in errors if error["loc"] == (field,)), None
                )
                if field_error is not None:
                    raise FixedValueValidationError(
                        "The value='{}' you are trying to use for field='{}' does not"
                        " pass the original validation with error={}".format(
                            value, field, field_error
                        )
                    )
            original = action_to_fix.parameters.__fields__[field]
            fixed[field] = (
                original.type_,
                Field(
                    value,
                    const=True,
                    description=original.field_info.description,
                    **original.field_info.extra,
                ),
            )
        with_fixed_parameters = create_model(
            "Fixed{}Parameters".format(action_type.name.capitalize()),
            __base__=action_to_fix.parameters,
            **fixed,
        )
        if action_type is ActionType.read:
            return Warehouse(
                name=self.name,
                data_schema=self.data_schema,
                data_type=self.data_type,
                read=WarehouseReadAction(
                    endpoints=self.read.endpoints,
                    parameters=with_fixed_parameters,
                    function=self.read.function,
                ),
                write=self.write,
            )

        return Warehouse(
            name=self.name,
            data_schema=self.data_schema,
            data_type=self.data_type,
            read=self.read,
            write=WarehouseWriteAction(
                endpoints=self.write.endpoints,
                parameters=with_fixed_parameters,
                function=self.write.function,
            ),
        )



from enum import Enum
from typing import List, Optional

from pydantic import BaseModel


class Kind(str, Enum):
    JOB = "job"
    TALENT_POOL = "talent_pool"


class RecruiteeProfile(BaseModel):
    name: str
    remote_cv_url: Optional[str]
    emails: Optional[List[str]]
    phones: Optional[List[str]]
    social_links: Optional[List[str]]
    links: Optional[List[str]]
    cover_letter: Optional[str]
    sources: Optional[List[str]]


class RecruiteeJob(BaseModel):
    title: str
    department: Optional[str]
    kind: Optional[Kind]
    description: str
    requirements: str
    postal_code: str
    city: str
    state_code: str
    country_code: str
    remote = False



import typing as t

from hrflow_connectors.connectors.greenhouse.warehouse import (
    GreenhouseJobWarehouse,
    GreenhouseProfileWarehouse,
)
from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.hrflow.warehouse.job import remove_html_tags
from hrflow_connectors.core import (
    ActionName,
    ActionType,
    BaseActionParameters,
    Connector,
    ConnectorAction,
    ConnectorType,
    WorkflowType,
)

APPLICATION_TAG = "application_boardKey_jobReference"


def format_job(data: t.Dict) -> t.Dict:
    """
    format each job pulled from greenhouse job board into a HrFlow job object
    Returns:
        HrflowJob: job in the HrFlow job object format
    """
    job = dict()
    # name
    job["name"] = data.get("title")
    # summary
    job["summary"] = None
    # reference
    job["reference"] = str(data.get("id"))
    # url
    job["url"] = data.get("absolute_url")
    # location
    location = data.get("location").get("name")
    job["location"] = dict(text=location, lat=None, lng=None)
    # sections
    description_content = data.get("content")
    # convert the escaped description content into html format
    # description_html = html.unescape(description_content)
    # remove html tags to get clean text
    text = remove_html_tags(description_content)

    job["sections"] = [
        dict(
            name="greenhouse_description",
            title="greenhouse_description",
            description=text,
        )
    ]
    # metadata
    job["metadatas"] = data.get("metadata")
    # tags
    department = data.get("departments")
    if department not in [None, []]:
        department_name = department[0].get("name")
        department_id = str(department[0].get("id"))
    else:
        department_name = "Undefined"
        department_id = "Undefined"

    office = data.get("offices")
    if office not in [None, []]:
        office_name = office[0].get("name")
        office_id = str(office[0].get("id"))
    else:
        office_name = "Undefined"
        office_id = "Undefined"

    education = data.get("education")
    employment = data.get("employment")

    job["tags"] = [
        dict(name="greenhouse_department-name", value=department_name),
        dict(name="greenhouse_department-id", value=department_id),
        dict(name="greenhouse_office-location", value=office_name),
        dict(name="greenhouse_office-id", value=office_id),
        dict(name="greenhouse_education", value=education),
        dict(name="greenhouse_employment", value=employment),
    ]
    # updated_at
    job["updated_at"] = data.get("updated_at")
    return job


def format_profile(data: t.Dict) -> t.Dict:
    """
    Format a profile hrflow object to a greenhouse profile object
    Args:
        profile (HrflowProfile): profile object in the hrflow profile format
    Returns:
        GreenhouseProfileModel: profile in the greenhouse candidate  format
    """
    profile = dict()
    profile["applications"] = []
    tags = data.get("tags")
    applications = list(filter(lambda x: x["name"] == APPLICATION_TAG, tags))
    job_id_list = list(map(lambda x: x["value"].split("_")[1], applications))
    for i in range(0, len(job_id_list)):
        job_id_list[i] = int(job_id_list[i])
    if len(job_id_list) == 0:
        raise Exception(
            "No job_id found, tag named '{}' either none existent or name poorly"
            " formated.".format(APPLICATION_TAG)
        )
    for id in job_id_list:
        profile["applications"].append(dict(job_id=id))

    profile["first_name"] = data.get("info").get("first_name")
    profile["last_name"] = data.get("info").get("last_name")
    profile["external_id"] = data.get("reference")

    if data.get("attachments") not in [[], None]:
        profile["resume"] = data.get("attachments")[0]["public_url"]

    phone_number = data.get("info").get("phone")
    profile["phone_numbers"] = [dict(value=phone_number, type="mobile")]

    email = data.get("info").get("email")
    profile["email_addresses"] = [dict(value=email, type="personal")]

    address = data.get("info").get("location").get("text")
    profile["addresses"] = [dict(value=address, type="home")]

    profile["notes"] = data.get("text")

    def get_social_media_urls():
        urls = data["info"]["urls"]
        website_list = []
        for url in urls:
            if isinstance(url, dict):
                if url["url"] not in ["", None, []]:
                    website_list.append(dict(value=url["url"]))
        return website_list

    if get_social_media_urls() not in [[], None]:
        profile["social_media_addresses"] = get_social_media_urls()

    if data["experiences"] not in [[], None]:
        last_experience = data["experiences"][0]
        profile["company"] = last_experience["company"]
        profile["title"] = last_experience["title"]
        profile["employments"] = []
        for experience in data["experiences"]:
            if (
                experience["title"]
                and experience["company"]
                and experience["date_start"]
            ) not in ["", None]:
                profile["employments"].append(
                    dict(
                        company_name=experience["company"],
                        title=experience["title"],
                        start_date=experience["date_start"],
                        end_date=experience["date_end"],
                    )
                )
    return profile


Greenhouse = Connector(
    name="Greenhouse",
    type=ConnectorType.ATS,
    description="",
    url="https://www.greenhouse.io/",
    actions=[
        ConnectorAction(
            name=ActionName.pull_job_list,
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs of a board via the ***Greenhouse*** API and send"
                " them to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "ReadJobsActionParameters", format=format_job
            ),
            origin=GreenhouseJobWarehouse,
            target=HrFlowJobWarehouse,
            action_type=ActionType.inbound,
        ),
        ConnectorAction(
            name=ActionName.push_profile,
            trigger_type=WorkflowType.catch,
            description=(
                "Writes a profile from Hrflow.ai Source to Greenhouse  via the API"
                " for the given job_id(s) provided in tags."
            ),
            parameters=BaseActionParameters.with_defaults(
                "WriteProfileActionParameters", format=format_profile
            ),
            origin=HrFlowProfileWarehouse,
            target=GreenhouseProfileWarehouse,
            action_type=ActionType.outbound,
        ),
    ],
)



import typing as t
from datetime import datetime

from pydantic import BaseModel, HttpUrl


class ContractDuration(BaseModel):
    min: t.Optional[int]
    max: t.Optional[int]


class Salary(BaseModel):
    min: t.Optional[int]
    max: t.Optional[int]
    kind: t.Optional[str]
    rate_type: t.Optional[str]
    variable: t.Optional[str]
    currency: t.Optional[str]


class AddressParts(BaseModel):
    street: str
    zip: str
    city: str
    county: str
    state: str
    country: str


class Address(BaseModel):
    parts: AddressParts
    formatted: str
    position: t.Dict[str, str]


class Manager(BaseModel):
    section_title: str
    section_body: str
    picture_url: t.Optional[str]
    firstname: str
    lastname: str
    position: str


class Hierarchy(BaseModel):
    depth: int
    column_name: str
    public_name: str


class Entity(BaseModel):
    public_name: str
    internal_ref: str
    around: str
    address: Address
    manager: Manager
    hierarchy: t.List[Hierarchy]


class ReferentRecruiter(BaseModel):
    firstname: str
    lastname: str
    picture_url: t.Optional[str]


class Brand(BaseModel):
    name: str
    description: str
    logo: str
    favicon: str


class CustomField(BaseModel):
    hash: str
    name: str
    value: str


class DigitalRecruitersJob(BaseModel):
    locale: str
    reference: str
    published_at: str
    catch_phrase: str
    contract_type: str
    contract_duration: ContractDuration
    contract_work_period: str
    service: str
    experience_level: str
    education_level: str
    title: str
    description: str
    profile: str
    skills: t.List[str]
    salary: Salary
    pictures: t.List[str]
    videos: t.List[str]
    internal_apply_url: t.Optional[str]
    apply_url: t.Optional[str]
    address: Address
    entity: Entity
    referent_recruiter: ReferentRecruiter
    brand: Brand
    custom_fields: t.List[CustomField]
    count_recruited: t.Optional[str]


class DigitalRecruitersCandidateProfile(BaseModel):
    gender: int
    firstName: str
    lastName: str
    email: str
    phoneNumber: t.Optional[str] = None
    job: t.Optional[str] = None
    addressStreet: t.Optional[str] = None
    addressZip: t.Optional[str] = None
    addressCity: t.Optional[str] = None


class DigitalRecruitersImportCandidateMessage(BaseModel):
    message: str


class DigitalRecruitersImportCandidateFile(BaseModel):
    content: str
    name: str


class DigitalRecruitersWriteProfile(BaseModel):
    reference: str
    consent_date: str
    s_o: str
    locale: str
    ApplicationMessage: DigitalRecruitersImportCandidateMessage
    ApplicationProfile: DigitalRecruitersCandidateProfile
    file: t.Optional[DigitalRecruitersImportCandidateFile] = None


class Location(BaseModel):
    zip: str
    city: str
    county: t.Optional[str]
    state: t.Optional[str]
    country: str
    latitude: t.Optional[float]
    longitude: t.Optional[float]


class ContractItem(BaseModel):
    id: int
    name: str
    countryNodeIds: t.Optional[t.List[int]]


class JobReference(BaseModel):
    label: str
    hashId: str


class Privacy(BaseModel):
    status: str
    updatedAt: t.Optional[datetime]


class Avatar(BaseModel):
    url: HttpUrl


class CV(BaseModel):
    url: HttpUrl


class Resume(BaseModel):
    raw: bytes
    content_type: str


class DigitalRecruitersReadProfile(BaseModel):
    id: int
    firstName: str
    lastName: str
    createdAt: datetime
    jobTitle: str
    avatar: Avatar
    gender: str
    email: str
    location: Location
    contract: ContractItem
    status: str
    jobReference: JobReference
    privacy: Privacy
    cv: CV
    resume: t.Optional[Resume]



import json
import os
import typing as t
from pathlib import Path

from pydantic import BaseModel

from hrflow_connectors.core.backend.common import BackendStore


class LocalJsonStore(BackendStore):
    DIRECTORY_ENVIRONMENT_VARIABLE = "HRFLOW_CONNECTORS_LOCALJSON_DIR"
    STORE_FILENAME = "store.json"

    def __init__(
        self,
    ) -> None:
        directory = os.environ.get(LocalJsonStore.DIRECTORY_ENVIRONMENT_VARIABLE)
        if directory is None:
            raise Exception(
                "Missing environment variable {} in"
                " order to setup LocalJson store".format(
                    LocalJsonStore.DIRECTORY_ENVIRONMENT_VARIABLE
                )
            )
        directory = Path(directory)
        if directory.is_absolute() is False:
            raise Exception(
                "{}={} should be an absolute filepath".format(
                    LocalJsonStore.DIRECTORY_ENVIRONMENT_VARIABLE, directory
                )
            )
        if directory.exists() is False:
            raise Exception(
                "{}={} does not exist".format(
                    LocalJsonStore.DIRECTORY_ENVIRONMENT_VARIABLE, directory
                )
            )
        self.store_fd = directory / LocalJsonStore.STORE_FILENAME
        if self.store_fd.exists() is False:
            self.store_fd.write_text(LocalJsonStore.dumps(LocalJsonStore.empty_store()))
        else:
            try:
                json.loads(self.store_fd.read_text())
            except (json.JSONDecodeError, UnicodeDecodeError):
                raise Exception("Store file is corrupted. Unable to JSON decode")

    @staticmethod
    def NAME() -> str:
        return "localjson"

    @staticmethod
    def empty_store() -> t.Dict:
        return dict(root="HrFlow Connectors", store=LocalJsonStore.NAME(), data=dict())

    @staticmethod
    def dumps(data: t.Any) -> str:
        return json.dumps(data, indent=2)

    def save(self, key: str, data: BaseModel) -> None:
        store = json.loads(self.store_fd.read_text())
        store["data"][key] = data.json()
        self.store_fd.write_text(LocalJsonStore.dumps(store))
        return None

    def load(self, key: str, parse_as: t.Type[BaseModel]) -> t.Optional[BaseModel]:
        store = json.loads(self.store_fd.read_text())
        if key in store["data"]:
            return parse_as.parse_raw(store["data"][key])
        return None



import time
import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field
from typing_extensions import Literal

from hrflow_connectors.connectors.lever.schemas import LeverJob, LeverProfile
from hrflow_connectors.core import (
    ActionEndpoints,
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)

LEVER_AUTH_ENDPOINT = "https://{auth_domain}.auth0.com"
LEVER_REDIRECT_URI = "https://marketplace-partners.hrflow.ai/partner/lever/login"
LEVER_BASE_URL = "https://{client_domain}.lever.co"
LEVER_JOBS_ENDPOINT = "https://{client_domain}.lever.co/v1/postings"
LEVER_OPPORTUNITIES_ENDPOINT = "https://{client_domain}.lever.co/v1/opportunities"

GET_ALL_JOBS_ENDPOINT = ActionEndpoints(
    name="Get all jobs",
    description="Endpoint to get the list of all jobs",
    url=LEVER_JOBS_ENDPOINT,
)
GET_ALL_PROFILES_ENDPOINT = ActionEndpoints(
    name="Get all profiles",
    description="Endpoint to get the list of all profiles",
    url=LEVER_OPPORTUNITIES_ENDPOINT,
)
POST_PROFILE_ENDPOINT = ActionEndpoints(
    name="Post Profile",
    description="Endpoint to create a new profile",
    url=LEVER_OPPORTUNITIES_ENDPOINT,
)


class LeverParameters(ParametersModel):
    auth_domain: str = Field(
        ...,
        description=(
            "Auth domain for authenticating with Lever API, exemple: sandbox-lever"
        ),
        field_type=FieldType.Auth,
    )
    client_domain: str = Field(
        ...,
        description=(
            "Client domain for authenticating with Lever API, exemple: api.sandbox"
        ),
        field_type=FieldType.Auth,
    )
    client_id: str = Field(
        ...,
        description="Client ID for authenticating with Lever API",
        field_type=FieldType.Auth,
    )
    client_secret: str = Field(
        ...,
        description="Client secret for authenticating with Lever API",
        field_type=FieldType.Auth,
    )
    authorization_code: str = Field(
        ...,
        description="Authorization code for obtaining access token",
        field_type=FieldType.Auth,
    )


class ReadParameters(LeverParameters):
    limit: int = Field(
        100,
        description="Number of jobs to fetch per request (max: 100)",
        field_type=FieldType.QueryParam,
    )


class ReadJobsParameters(ReadParameters):
    pass


class WriteProfileParameters(LeverParameters):
    perform_as: str = Field(
        ...,
        description="User ID on behalf of whom the create action should be performed",
        field_type=FieldType.QueryParam,
    )
    parse: bool = Field(
        False,
        description="If true, parse resume for autofilling",
        field_type=FieldType.QueryParam,
    )
    perform_as_posting_owner: bool = Field(
        False,
        description="If true, set Opportunity owner to posting owner",
        field_type=FieldType.QueryParam,
    )


class ReadProfilesParameters(ReadParameters):
    pass


def get_or_refresh_tokens(
    auth_domain,
    client_id,
    client_secret,
    flow: Literal["authorization_code", "refresh_token"],
    code: str,
):
    lever_auth_endpoint = LEVER_AUTH_ENDPOINT.format(auth_domain=auth_domain)
    url = "{}/oauth/token".format(lever_auth_endpoint)
    redirect_uri = LEVER_REDIRECT_URI
    if flow == "authorization_code":
        request_data = {
            "client_id": client_id,
            "client_secret": client_secret,
            "grant_type": flow,
            "code": code,
            "redirect_uri": redirect_uri,
        }
    else:
        request_data = {
            "client_id": client_id,
            "client_secret": client_secret,
            "grant_type": flow,
            "refresh_token": code,
        }

    response = requests.post(url, data=request_data)
    if response.status_code == 200:
        response_data = response.json()
        access_token = response_data.get("access_token")
        new_refresh_token = response_data.get("refresh_token")
        return access_token, new_refresh_token

    raise Exception(
        f"Failed to obtain token. Status code: {response.status_code},"
        f" Response: {response.text}"
    )


def read_jobs(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    token, refresh_token = get_or_refresh_tokens(
        parameters.auth_domain,
        parameters.client_id,
        parameters.client_secret,
        "authorization_code",
        parameters.authorization_code,
    )
    jobs_url = LEVER_JOBS_ENDPOINT.format(client_domain=parameters.client_domain)
    offset = None
    while True:
        headers = {"Authorization": "Bearer " + token}
        params = {
            "limit": parameters.limit,
        }
        if offset is not None:
            params["offset"] = offset
        response = requests.get(jobs_url, headers=headers, params=params)
        if response.status_code == 200:
            jobs = response.json()["data"]
            for job in jobs:
                yield job
            if response.json().get("hasNext", False):
                offset = response.json().get("next", None)
            else:
                break
        elif response.status_code == 429:
            adapter.warning("Rate limit exceeded. Retrying after 5 second.")
            time.sleep(5)

        elif response.status_code == 401:
            adapter.warning("Access token has expired. Refreshing token and retrying.")
            token, refresh_token = get_or_refresh_tokens(
                parameters.auth_domain,
                parameters.client_id,
                parameters.client_secret,
                "refresh_token",
                refresh_token,
            )
        else:
            raise Exception(
                "Failed to retrieve jobs from Lever. Status code:"
                f" {response.status_code}, Response: {response.text}"
            )


def read_profiles(
    adapter: LoggerAdapter,
    parameters: ReadProfilesParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    token, refresh_token = get_or_refresh_tokens(
        parameters.auth_domain,
        parameters.client_id,
        parameters.client_secret,
        "authorization_code",
        parameters.authorization_code,
    )
    offset = None
    profiles_url = LEVER_OPPORTUNITIES_ENDPOINT.format(
        client_domain=parameters.client_domain
    )
    while True:
        headers = {"Authorization": "Bearer " + token}
        params = {
            "limit": parameters.limit,
        }
        if offset is not None:
            params["offset"] = offset
        response = requests.get(profiles_url, headers=headers, params=params)
        if response.status_code == 200:
            opportunities = response.json()["data"]
            for opportunity in opportunities:
                opportunity_id = opportunity["id"]
                profile_response = requests.get(
                    f"{profiles_url}/{opportunity_id}/resumes",
                    headers=headers,
                )
                if profile_response.status_code == 200:
                    profile_data = profile_response.json()
                    opportunity["profile"] = profile_data["data"]
                    yield opportunity
                else:
                    raise Exception(
                        "Failed to retrieve profiles for opportunity"
                        f" {opportunity_id}. Status code:"
                        f" {profile_response.status_code}, Response:"
                        f" {profile_response.text}"
                    )

            if response.json().get("hasNext", False):
                offset = response.json().get("next", None)
            else:
                break

        elif response.status_code == 429:
            adapter.warning("Rate limit exceeded. Retrying after 1 minute.")
            time.sleep(5)
        elif response.status_code == 401:
            adapter.warning("Access token has expired. Refreshing token and retrying.")
            token, refresh_token = get_or_refresh_tokens(
                parameters.auth_domain,
                parameters.client_id,
                parameters.client_secret,
                "refresh_token",
                refresh_token,
            )
        else:
            raise Exception(
                "Failed to retrieve profiles from Lever. Status code:"
                f" {response.status_code}, Response: {response.text}"
            )


def write(
    adapter: LoggerAdapter,
    parameters: WriteProfileParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    token, refresh_token = get_or_refresh_tokens(
        parameters.auth_domain,
        parameters.client_id,
        parameters.client_secret,
        "authorization_code",
        parameters.authorization_code,
    )
    url_post_opportunity = LEVER_OPPORTUNITIES_ENDPOINT.format(
        client_domain=parameters.client_domain
    )
    failed_profiles = []
    params = {
        "perform_as": parameters.perform_as,
        "parse": parameters.parse,
        "perform_as_posting_owner": parameters.perform_as_posting_owner,
    }
    i = 0
    while i < len(profiles):
        profile = profiles[i]
        profile.pop("file", None)
        headers = {"Authorization": "Bearer " + token}
        response = requests.post(
            url_post_opportunity,
            headers=headers,
            params=params,
            json=profile,
        )
        if response.status_code // 100 == 2:
            adapter.info("Successfully posted profile")
            i += 1
        elif response.status_code == 429:
            adapter.warning("Rate limit exceeded. Retrying after 1 minute.")
            time.sleep(5)
        elif response.status_code == 401:
            adapter.warning("Access token has expired. Refreshing token and retrying.")
            token, refresh_token = get_or_refresh_tokens(
                parameters.auth_domain,
                parameters.client_id,
                parameters.client_secret,
                "refresh_token",
                refresh_token,
            )
        else:
            failed_profiles.append(profile)
            adapter.error(
                "Failed to post profile. Status code: %s, Response: %s",
                response.status_code,
                response.text,
            )

    return failed_profiles


LeverJobWarehouse = Warehouse(
    name="Lever Jobs",
    data_schema=LeverJob,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read_jobs,
        endpoints=[GET_ALL_JOBS_ENDPOINT],
    ),
)

LeverProfileWarehouse = Warehouse(
    name="Lever Profiles",
    data_schema=LeverProfile,
    data_type=DataType.profile,
    read=WarehouseReadAction(
        parameters=ReadProfilesParameters,
        function=read_profiles,
        endpoints=[GET_ALL_PROFILES_ENDPOINT],
    ),
    write=WarehouseWriteAction(
        parameters=WriteProfileParameters,
        function=write,
        endpoints=[POST_PROFILE_ENDPOINT],
    ),
)



import enum
import logging
import os
import re
import subprocess
import typing as t
from contextvars import ContextVar
from datetime import datetime
from pathlib import Path

from pydantic import BaseModel
from pydantic.fields import ModelField

from hrflow_connectors.core import ActionName
from hrflow_connectors.core.connector import Connector
from hrflow_connectors.core.templates import Templates

logger = logging.getLogger(__name__)
CONNECTORS_DIRECTORY = Path(__file__).parent.parent / "connectors"

DONE_MARKUP = ":white_check_mark:"
KO_MARKUP = ":x:"


ROOT_README_TRACKED_ACTIONS = {
    ActionName.pull_job_list,
    ActionName.pull_profile_list,
    ActionName.push_profile,
    ActionName.push_job,
}


def CONNECTOR_LISTING_REGEXP_F(name: str) -> str:
    return (
        r"\|\s*\[?\*{0,2}(?i:(?P<name>"
        + r" ?".join([c for c in name if c.strip()])
        + r"))\*{0,2}(\]\([^)]+\))?\s*\|[^|]+\|[^|]+\|\s*(\*|_)(?P<release_date>[\d\/]+)(\*|_)\s*\|.+"
    )


ACTIONS_SECTIONS_REGEXP = (
    r"# 🔌 Connector Actions.+?\|\s*Action\s*\|\s*Description\s*\|.+?\|\s+?<\/p>"
)


GIT_UPDATE_TIMEOUT = 5
GIT_UPDATE_DATE = """
git ls-tree -r --name-only HEAD {base_connector_path}/{connector} | while read filename; do
  echo "$(git log -1 --format="%aI" -- $filename)"
done
"""

HRFLOW_CONNECTORS_REMOTE_URL = "https://github.com/Riminder/hrflow-connectors"
USE_REMOTE_REV: ContextVar[t.Optional[str]] = ContextVar("USE_REMOTE_REV", default=None)
BASE_CONNECTOR_PATH: ContextVar[t.Optional[str]] = ContextVar(
    "BASE_CONNECTOR_PATH", default="src/hrflow_connectors/connectors/"
)


class InvalidConnectorReadmeFormat(Exception):
    pass


class TemplateField(BaseModel):
    name: str
    type: str
    required: bool
    description: str
    example: str
    default: str


def field_example(field: ModelField) -> str:
    if callable(field.default):
        return "lambda *args, **kwargs: None # Put your code logic here"

    if field.default is not None:
        if isinstance(field.default, str):
            return '"{}"'.format(field.default)
        return str(field.default)

    if field.default_factory is not None:
        return str(field.default_factory())

    field_type = field.outer_type_
    if isinstance(field_type, enum.EnumMeta):
        return '"{}"'.format(list(field_type)[0].value)

    if field_type is str:
        return '"your_{}"'.format(field.name)

    if field_type in [int, float, bool]:
        return str(field_type())

    return "***"


def field_default(field: ModelField, documentation_path: Path) -> str:
    if callable(field.default):
        filepath = os.path.relpath(
            field.default.__code__.co_filename, documentation_path
        )
        if (
            "site-packages/hrflow_connectors/" in filepath
            and USE_REMOTE_REV.get() is not None
        ):
            filepath = "{}/tree/{}/src/hrflow_connectors/{}".format(
                HRFLOW_CONNECTORS_REMOTE_URL,
                USE_REMOTE_REV.get(),
                filepath.split("/hrflow_connectors/")[-1],
            )
        return "[`{}`]({}#L{})".format(
            field.default.__code__.co_name,
            filepath,
            field.default.__code__.co_firstlineno,
        )

    if field.default_factory is not None:
        return str(field.default_factory())

    return str(field.default)


def field_type(field: ModelField) -> str:
    if field.outer_type_ in [int, float, str, bool]:
        return field.outer_type_.__name__
    if isinstance(field.outer_type_, enum.EnumMeta):
        return "str"
    return str(field.outer_type_)


def get_template_fields(
    fields: t.List[ModelField], documentation_path: Path
) -> t.List[TemplateField]:
    return [
        TemplateField(
            name=field.name,
            type=field_type(field),
            required=field.required,
            description=field.field_info.description or "",
            example=field_example(field),
            default=field_default(field, documentation_path),
        )
        for field in fields
        if not field.field_info.const
        and field.field_info.extra.get("skip_from_docs", False) is False
    ]


def py_37_38_compat_patch(content: str) -> str:
    """
    The way t.Optional[T] is stringified is different accross supported python versions:
        - Python 3.7, 3.8 --> typing.Union[T, NoneType]
        - Python >= 3.9 --> t.Optional[T]
    This creates inconsistency when generating the doc with accross these versions.
    This function changes any older string versions to match with >=3.9
    """
    return re.sub(
        r"Union\[([\w\.]+), NoneType\]",
        lambda match: f"Optional[{match.group(1)}]",
        content,
    )


def update_root_readme(connectors: t.List[Connector], root: Path) -> t.Dict:
    readme = root / "README.md"
    if readme.exists() is False:
        raise Exception("Failed to find root README.md at {}".format(readme))
    readme_content = readme.read_text()

    for connector in connectors:
        model = connector.model
        result = subprocess.run(
            GIT_UPDATE_DATE.format(
                connector=model.name.lower(),
                base_connector_path=BASE_CONNECTOR_PATH.get().rstrip("/"),
            ),
            shell=True,
            text=True,
            capture_output=True,
            timeout=GIT_UPDATE_TIMEOUT,
        )
        if result.stderr:
            raise Exception(
                "Subprocess run for Git update dates failed for connector {} with"
                " errors {}".format(model.name.lower(), result.stderr)
            )
        updated_at = datetime.fromisoformat(
            max(
                result.stdout.strip().splitlines(),
                key=lambda d: datetime.fromisoformat(d),
            )
        )
        actions_status = dict(
            zip(
                ROOT_README_TRACKED_ACTIONS,
                [KO_MARKUP] * len(ROOT_README_TRACKED_ACTIONS),
            )
        )
        for action in model.actions:
            if action.name in ROOT_README_TRACKED_ACTIONS:
                actions_status[action.name] = DONE_MARKUP

        pattern = CONNECTOR_LISTING_REGEXP_F(name=model.name)
        match = re.search(pattern, readme_content)
        if match is None:
            raise Exception(
                "Could not find listing for {} in root README with regexp={}".format(
                    model.name, pattern
                )
            )
        updated_listing = (
            "| [**{name}**]({readme_link}) | {type} | :white_check_mark: |"
            " *{release_date}* | *{updated_at}* | {pull_profile_list_status} |"
            " {pull_job_list_status} | {push_profile_status} | {push_job_status} |"
        ).format(
            name=match.group("name"),
            readme_link="./{base_connector_path}/{connector}/README.md".format(
                base_connector_path=BASE_CONNECTOR_PATH.get().strip("/"),
                connector=model.name.lower(),
            ),
            type=model.type.value,
            release_date=match.group("release_date"),
            updated_at=updated_at.strftime("%d/%m/%Y"),
            pull_profile_list_status=actions_status[ActionName.pull_profile_list],
            pull_job_list_status=actions_status[ActionName.pull_job_list],
            push_profile_status=actions_status[ActionName.push_profile],
            push_job_status=actions_status[ActionName.push_job],
        )
        readme_content = (
            readme_content[: match.start()]
            + updated_listing
            + readme_content[match.end() :]
        )
    readme.write_bytes(readme_content.encode())


def generate_docs(
    connectors: t.List[Connector], connectors_directory: Path = CONNECTORS_DIRECTORY
) -> None:
    update_root_readme(
        connectors=connectors, root=connectors_directory.parent.parent.parent
    )
    for connector in connectors:
        model = connector.model
        connector_directory = connectors_directory / model.name.lower()
        if not connector_directory.is_dir():
            logging.error(
                "Skipping documentation for {}: no directory found at {}".format(
                    model.name, connector_directory
                )
            )
            continue
        readme = connector_directory / "README.md"
        if readme.exists() is False:
            readme_content = Templates.get_template("connector_readme.md.j2").render(
                connector_name=model.name.capitalize(),
                description=model.description,
                url=model.url,
                actions=model.actions,
            )
            readme_content = py_37_38_compat_patch(readme_content)
            readme.write_bytes(readme_content.encode())
        else:
            readme_content = readme.read_text()
            match = re.search(ACTIONS_SECTIONS_REGEXP, readme_content, re.DOTALL)
            if match is None:
                raise InvalidConnectorReadmeFormat(
                    "README.md for connector {} does not respect standard format. No"
                    " actions section found".format(model.name)
                )
            updated_actions_content = Templates.get_template(
                "connector_actions.md.j2"
            ).render(
                actions=model.actions,
            )
            updated_readme_content = "{before}{actions}{after}".format(
                before=readme_content[: match.start()],
                actions=updated_actions_content,
                after=readme_content[match.end() :],
            )
            updated_readme_content = py_37_38_compat_patch(updated_readme_content)
            readme.write_bytes(updated_readme_content.encode())
        if len(model.actions) > 0:
            action_docs_directory = connector_directory / "docs"
            if not action_docs_directory.is_dir():
                action_docs_directory.mkdir()
            for action in model.actions:
                action_name = action.name.value
                action_fields = get_template_fields(
                    fields=action.parameters.__fields__.values(),
                    documentation_path=action_docs_directory,
                )
                origin_fields = get_template_fields(
                    fields=action.origin.read.parameters.__fields__.values(),
                    documentation_path=action_docs_directory,
                )
                target_fields = get_template_fields(
                    fields=action.target.write.parameters.__fields__.values(),
                    documentation_path=action_docs_directory,
                )
                action_documentation_content = Templates.get_template(
                    "action_readme.md.j2"
                ).render(
                    connector_name=model.name,
                    action_name=action_name,
                    description=action.description,
                    action_fields=action_fields,
                    origin_name=action.origin.name,
                    origin_fields=origin_fields,
                    origin_endpoints=action.origin.read.endpoints,
                    target_name=action.target.name,
                    target_fields=target_fields,
                    target_endpoints=action.target.write.endpoints,
                )
                action_documentation_content = py_37_38_compat_patch(
                    action_documentation_content
                )
                action_documentation = action_docs_directory / "{}.md".format(
                    action_name
                )
                action_documentation.write_bytes(action_documentation_content.encode())



import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.ceridian.schemas import CeridianDayforceJobModel
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
)


class ReadJobsParameters(ParametersModel):
    subdomain: str = Field(
        ...,
        description="Subdomain used to access Ceridian API",
        repr=False,
        field_type=FieldType.Other,
    )

    client_name_space: str = Field(
        ...,
        description="Client name space used to access Ceridian API",
        repr=False,
        field_type=FieldType.Other,
    )

    companyName: t.Optional[str] = Field(
        None,
        description="Company name. Example: XYZ Co.",
        field_type=FieldType.QueryParam,
    )
    parentCompanyName: t.Optional[str] = Field(
        None,
        description="Parent Company name. Example: Ceridian",
        field_type=FieldType.QueryParam,
    )

    lastUpdateTimeFrom: t.Optional[str] = Field(
        None,
        description=(
            "A starting timestamp of job posting date. Example: 2017-01-01T13:24:56"
        ),
        field_type=FieldType.QueryParam,
    )

    htmlDescription: t.Optional[bool] = Field(
        None,
        description=(
            "A flag to feed the jobs over with html formatting or plain text"
            " description"
        ),
        field_type=FieldType.QueryParam,
    )


def read(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    params = dict()
    params["companyName"] = parameters.companyName
    params["parentCompanyName"] = parameters.parentCompanyName
    params["lastUpdateTimeFrom"] = parameters.lastUpdateTimeFrom
    params["htmlDescription"] = parameters.htmlDescription

    url = "https://{}.dayforcehcm.com/Api/{}/V1/JobFeeds".format(
        parameters.subdomain, parameters.client_name_space
    )

    response = requests.request(
        "GET",
        url=url,
        headers={},
        data={},
        params=params,
    )
    if response.status_code // 100 != 2:
        adapter.error(
            "Failed to pull jobs from Ceridian params={}"
            " status_code={} response={}".format(
                params, response.status_code, response.text
            )
        )
        raise Exception("Failed to pull jobs from Ceridian")
    response = response.json()
    return response


CeridianJobWarehouse = Warehouse(
    name="Ceridian Jobs",
    data_schema=CeridianDayforceJobModel,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read,
        endpoints=[],
    ),
)



# Code from PR: https://github.com/Riminder/hrflow-connectors/pull/66
# Author: Yasser BELMEHDI (yass1337)
import json
import typing as t
from datetime import datetime

from hrflow_connectors.connectors.hrflow.warehouse import (
    HrFlowJobWarehouse,
    HrFlowProfileWarehouse,
)
from hrflow_connectors.connectors.taleez.warehouse import (
    TaleezJobWarehouse,
    TaleezProfilesWarehouse,
)
from hrflow_connectors.core import (
    ActionName,
    ActionType,
    BaseActionParameters,
    Connector,
    ConnectorAction,
    ConnectorType,
    WorkflowType,
)
from hrflow_connectors.core.warehouse import ActionEndpoints

# TODO: These are examples custom properties
# --> Should be added as parameters to configure for each customer
PROPERTIES_PATH = "src/hrflow_connectors/connectors/taleez/properties.json"
DATE_FORMAT = "%Y-%m-%d"
INITIALREF = "HrFlow"
HRFLOW_TAGS_MAPPING = {
    "experience": 51933,
    "fr_education_level": 51934,
    "availability": 51935,
    "salary_wanted": 51936,
    "candidate_profile": 51937,
    "fr_contract": 51938,
    "CV": 51939,
    "source_candidate": 51940,
    "en_education_level": 56258,
    "candidate_contract": 56259,
    "last_position": 56263,
    "hard_skills": 56260,
    "languages": 56261,
    "courses": 56262,
    "last_education": 59898,
}

DEFAULT_CANDIDATE_SOURCE = "HrFlow"

EDUCATION_LEVELS_MAPPING = {
    600944: "Aucun diplôme",
    600945: "BEP, CAP ou équivalent",
    600946: "Baccalauréat (bac +0)",
    600947: "DUT, BTS, Diplôme d'état ou équivalent (bac +2)",
    600948: "Licence, Bachelor ou équivalent (bac +3)",
    600949: "Master 1 ou équivalent (bac +4)",
    600950: "Master, Diplôme d'ingénieur ou équivalent (bac +5)",
    600951: "Doctorat et équivalent (> bac +5)",
}

SOCIAL_LINKS = [
    "linkedin",
    "viadeo",
    "twitter",
    "github",
    "behance",
    "other",
    "website",
    "dribble",
]

TALEEZ_JOBS_ENDPOINT = "https://api.taleez.com/0/jobs"
TALEEZ_JOBS_ENDPOINT_LIMIT = 100


GET_ALL_JOBS_ENDPOINT = ActionEndpoints(
    name="Get all jobs",
    description=(
        "Endpoint to retrieve all jobs."
        " and get the list of all jobs with their ids, the request method"
        " is `GET`"
    ),
    url="https://api.taleez.com/0/jobs",
)


def get_education_level(profile: t.Dict):
    fr_education_level = []
    for education in profile["educations"]:
        for id in EDUCATION_LEVELS_MAPPING.keys():
            for keyword in EDUCATION_LEVELS_MAPPING[id]:
                if keyword in education["title"]:
                    fr_education_level.append(id)
    return fr_education_level


def get_CV(profile: t.Dict):
    return next(
        filter(lambda x: x.get("type") == "resume", profile["attachments"]), {}
    ).get("public_url")


def get_property_by_id(
    properties_list: t.List[t.Dict], prop_id: int
) -> t.Optional[t.Dict]:
    return next(iter(filter(lambda x: x["id"] == prop_id, properties_list)), None)


def tags_to_properties_mapping(properties: t.List[t.Dict]) -> t.Dict:
    mapping = dict()
    for tag_name, id_ in HRFLOW_TAGS_MAPPING.items():
        p = get_property_by_id(properties, id_)
        mapping[tag_name] = {"id": id_, "key": p["key"], "type": p["type"]}

    return mapping


class Property:
    def __init__(
        self,
        id: int,
        key: str,
        name: str,
        nameEn: str,
        type: str,
        choices: t.List[t.Dict],
    ) -> None:
        self.id = id
        self.key = key
        self.name = name
        self.nameEn = nameEn
        self.type = type
        self.choices = choices

    def get_property_choice_from_text(
        self, input_text: str, processor: t.Callable[[str], t.List[int]]
    ) -> t.List[int]:
        return processor(input_text)

    def generate_property_payload(
        self,
        input_text: t.Union[str, int],
        processor: t.Optional[t.Callable[[str], t.List[int]]] = None,
    ) -> t.Dict:
        payload = None
        if self.type in ["INPUT", "TEXTAREA", "DATE"]:
            payload = {"id": self.id, "value": input_text}
        elif self.type == "SELECT":
            assert processor is not None, (
                "for SELECT type Property a processor function to select choice id must"
                " be provided"
            )
            choices = self.get_property_choice_from_text(input_text, processor)
            assert len(choices) <= 1, (
                "for SELECT type Property, at most one choice must be selected. check"
                " processor function."
            )
            payload = {"id": self.id, "choices": choices}
        elif self.type == "SELECTMUL":
            assert processor is not None, (
                "for SELECTMUL type Property a processor function to select choice id"
                " must be provided"
            )
            choices = self.get_property_choice_from_text(input_text, processor)
            payload = {"id": self.id, "choices": choices}
        return payload


def get_parsed_hardskills(profile: t.Dict):
    skills = list(filter(lambda x: x["type"] == "hard", profile["skills"]))
    skills = [skill["name"] for skill in skills] if skills else []
    return ", ".join(skills) if skills else ""


def get_languages(profile: t.Dict):
    return (
        ", ".join([language["name"] for language in profile["languages"]])
        if profile["languages"]
        else ""
    )


def get_courses(profile: t.Dict):
    return (
        ", ".join([course["name"] for course in profile["courses"]])
        if profile["courses"]
        else ""
    )


def get_last_position(profile: t.Dict):
    return profile["experiences"][0]["title"]


def get_hrflow_tag_by_name(hrflow_profile: t.Dict, tag_name: str) -> t.Optional[t.Dict]:
    return next(filter(lambda x: x["name"] == tag_name, hrflow_profile["tags"]), {})


def get_profile_properties_to_push(
    profile: t.Dict, origin_properties: t.List[t.Dict]
) -> t.List[t.Dict]:
    """Takes hrflow profile object and the list of all properties
        from Taleez candidates endpoint
        Generates Property objects to leverage class methods
        to format tag values and each tag specific
        processor to format the text if needed or
        select the corresponding choice from choices list in case
        of a select field depending on the rule fixed in the processor function.
    Args:
        profile (t.Dict): HrFlow.ai Profile
        origin_properties (t.List[t.Dict]): List of properties
        created by the customer on Taleez (result of get candidate-properties)
    Returns:
        t.List[t.Dict]: List of properties payload
        to post via candidate-properties endpoint
    """
    output = []
    property_objects = dict()
    for tag_name, property_id in HRFLOW_TAGS_MAPPING.items():
        property_dict = get_property_by_id(origin_properties, property_id)
        if property_dict:
            property_objects[tag_name] = Property(**property_dict)

    tag_name = "source_candidate"
    input_text = profile["source"]["name"]
    output.append(
        property_objects[tag_name].generate_property_payload(input_text=input_text)
    )

    tag_name = "hard_skills"
    input_text = get_parsed_hardskills(profile)
    output.append(
        property_objects[tag_name].generate_property_payload(input_text=input_text)
    )

    tag_name = "languages"
    input_text = get_languages(profile)
    output.append(
        property_objects[tag_name].generate_property_payload(input_text=input_text)
    )

    tag_name = "courses"
    input_text = get_courses(profile)
    output.append(
        property_objects[tag_name].generate_property_payload(input_text=input_text)
    )

    tag_name = "last_position"
    input_text = profile["experiences"][0]["title"] if profile["experiences"] else ""
    output.append(
        property_objects[tag_name].generate_property_payload(input_text=input_text)
    )

    tag_name = "last_education"
    input_text = profile["educations"][0]["title"] if profile["educations"] else ""
    output.append(
        property_objects[tag_name].generate_property_payload(input_text=input_text)
    )

    return output


def format_profile(profile: t.Dict) -> t.Dict:
    social_links = {}
    for link in profile["info"]["urls"]:
        if link["type"] in SOCIAL_LINKS:
            social_links["{}".format(link["type"])] = link["url"]
    candidate = dict(
        firstName=profile["info"]["first_name"],
        lastName=profile["info"]["last_name"],
        mail=profile["info"]["email"],
        initialReferrer=INITIALREF,
        lang=str(profile["text_language"]).upper(),
        social_links=social_links,
    )
    cv = get_CV(profile)
    with open(PROPERTIES_PATH) as f:
        properties = get_profile_properties_to_push(profile, json.load(f)["list"])
    return dict(candidate=candidate, CV=cv, properties=properties)


def get_job_location(taleez_job: t.Union[t.Dict, None]) -> t.Dict:
    if taleez_job is None:
        return dict(lat=None, lng=None, text="")

    lat = taleez_job.get("lat")
    lat = float(lat) if lat is not None else lat

    lng = taleez_job.get("lng")
    lng = float(lng) if lng is not None else lng

    concatenate = []
    for field in ["postalCode", "city", "country"]:
        if taleez_job.get(field):
            concatenate.append(taleez_job.get(field))

    return dict(lat=lat, lng=lng, text=" ".join(concatenate))


def get_sections(taleez_job: t.Dict) -> t.List[t.Dict]:
    sections = []
    for section_name in ["jobDescription", "profileDescription", "companyDescription"]:
        section = taleez_job.get(section_name)
        if section is not None:
            sections.append(
                dict(
                    name="taleez-sections-{}".format(section_name),
                    title=section_name,
                    description=section,
                )
            )
    return sections


def get_tags(taleez_job: t.Dict) -> t.List[t.Dict]:
    taleez_tags = taleez_job.get("tags")
    if taleez_tags is None:
        return []
    t = lambda name, value: dict(name=name, value=value)

    custom_fields = [
        "contract",
        "profile",
        "urlApplying",
        "currentStatus",
    ]

    tags = [
        t("{}_{}".format("taleez", field), taleez_job.get(field))
        for field in custom_fields
    ]
    tags += [t("{}_{}".format("taleez", "tag"), tag) for tag in taleez_tags]
    return tags


def format_job(taleez_job: t.Dict) -> t.Dict:
    job = dict(
        name=taleez_job.get("label", "Undefined"),
        reference=str(taleez_job.get("id")),
        created_at=datetime.fromtimestamp(taleez_job["dateCreation"]).isoformat(),
        updated_at=datetime.fromtimestamp(taleez_job["dateLastPublish"]).isoformat(),
        location=get_job_location(taleez_job),
        url=taleez_job.get("url"),
        summary=None,
        sections=get_sections(taleez_job),
        tags=get_tags(taleez_job),
    )
    return job


DESCRIPTION = (
    "Taleez est une solution globale de gestion des candidatures et de diffusion"
    " d'offres d'emploi.Pilotez intégralement vos processus de recrutement et intégrez"
    " vos équipes dans les décisions."
)

Taleez = Connector(
    name="Taleez",
    type=ConnectorType.ATS,
    description=DESCRIPTION,
    url="https://taleez.com/",
    actions=[
        ConnectorAction(
            name=ActionName.push_profile,
            trigger_type=WorkflowType.catch,
            description=(
                "Retrieves a profile from HrFlow Source and posts it to Taleez ATS "
                "enriching it with properties extracted from the profile"
            ),
            parameters=BaseActionParameters.with_defaults(
                # FIXME : why is it called ReadJobsActionParameters ?
                "ReadJobsActionParameters",
                format=format_profile,
            ),
            origin=HrFlowProfileWarehouse,
            target=TaleezProfilesWarehouse,
            action_type=ActionType.outbound,
        ),
        ConnectorAction(
            name=ActionName.pull_job_list,
            trigger_type=WorkflowType.pull,
            description=(
                "Retrieves all jobs via the ***Taleez*** API and send them"
                " to a ***Hrflow.ai Board***."
            ),
            parameters=BaseActionParameters.with_defaults(
                "PullJobsActionParameters", format=format_job
            ),
            origin=TaleezJobWarehouse,
            target=HrFlowJobWarehouse,
            action_type=ActionType.inbound,
        ),
    ],
)



import typing as t

from pydantic import BaseModel


class Property(BaseModel):
    id: int
    key: str
    name: str
    nameEn: str
    type: str
    choices: t.List[t.Dict]


class Candidate(BaseModel):
    id: int
    firstName: str
    lastName: str
    mail: str
    initialReferrer: str
    lang: str
    socialLinks: dict
    properties: t.Optional[list]
    jobs: t.Optional[list]


class Job(BaseModel):
    id: int
    token: str
    dateCreation: int
    dateFirstPublish: int
    dateLastPublish: int
    label: str
    currentStatus: str
    contract: str
    contractLength: int
    fullTime: bool
    workHours: int
    remote: bool
    country: str
    city: str
    postalCode: str
    lat: str
    lng: str
    recruiterId: int
    who: str
    logo: str
    banner: str
    companyLabel: str
    tags: t.List[t.Dict]
    url: str
    urlApplying: str
    jobDescription: str
    profileDescription: str
    companyDescription: str
    properties: t.List[t.Dict]
    public: bool


class JobProperty(BaseModel):
    key: str
    value: str
    values: t.List[str]



import typing as t
from enum import Enum
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.adzuna.schemas import AdzunaJob
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
)
from hrflow_connectors.core.warehouse import ActionEndpoints

ADZUNA_ENDPOINT = "http://api.adzuna.com/v1/api"
SEARCH_JOBS_ENDPOINT = ActionEndpoints(
    name="Get Adzuna jobs",
    description="Use this endpoint to retrieve Adzuna's job advertisement listings.",
    url="https://api.adzuna.com/v1/doc/Search.md",
)

MAX_NUMBER_OF_PAGES = 10


class CountryCode(str, Enum):
    GB = "gb"
    AT = "at"
    AU = "au"
    BR = "br"
    CA = "ca"
    DE = "de"
    FR = "fr"
    IN = "in"
    IT = "it"
    NL = "nl"
    NZ = "nz"
    PL = "pl"
    RU = "ru"
    SG = "sg"
    US = "us"
    ZA = "za"


class SortDir(str, Enum):
    UP = "up"
    DOWN = "down"


class SortKey(str, Enum):
    DEFAULT = "default"
    HYBRID = "hybrid"
    DATE = "date"
    SALARY = "salary"
    RELEVANCE = "relevance"


class Filter(str, Enum):
    ON = "1"


class ReadJobsParameters(ParametersModel):
    country: CountryCode = Field(
        ...,
        description="ISO 8601 country code of the country of interest",
        field_type=FieldType.QueryParam,
    )
    app_id: str = Field(
        ...,
        description="Application ID, supplied by Adzuna",
        repr=False,
        field_type=FieldType.Auth,
    )
    app_key: str = Field(
        ...,
        description="Application key, supplied by Adzuna",
        repr=False,
        field_type=FieldType.Auth,
    )
    results_per_page: int = Field(
        None,
        description="The number of results to include on a page of search results.",
        field_type=FieldType.QueryParam,
    )
    what: str = Field(
        None,
        description=(
            "The keywords to search for. Multiple terms may be space separated."
        ),
        field_type=FieldType.QueryParam,
    )
    what_and: str = Field(
        None,
        description="The keywords to search for, all keywords must be found.",
        field_type=FieldType.QueryParam,
    )
    what_phrase: str = Field(
        None,
        description="An entire phrase which must be found in the description or title.",
        field_type=FieldType.QueryParam,
    )
    what_or: str = Field(
        None,
        description=(
            "The keywords to search for, any keywords may be found. Multiple terms may"
            " be space separated."
        ),
        field_type=FieldType.QueryParam,
    )
    what_exclude: str = Field(
        None,
        description=(
            "Keywords to exclude from the search. Multiple terms may be space"
            " separated."
        ),
        field_type=FieldType.QueryParam,
    )
    title_only: str = Field(
        None,
        description=(
            "Keywords to find, but only in the title. Multiple terms may be space"
            " separated."
        ),
        field_type=FieldType.QueryParam,
    )
    where: str = Field(
        None,
        description=(
            "The geographic centre of the search. Place names, postal codes, etc. may"
            " be used.	"
        ),
        field_type=FieldType.QueryParam,
    )
    distance: int = Field(
        None,
        description=(
            "The distance in kilometres from the centre of the place described by the"
            " 'where' parameter. Defaults to 5km."
        ),
        field_type=FieldType.QueryParam,
    )
    location0: str = Field(
        None,
        description=(
            """The location fields may be used to describe a location,"""
            """ in a similar form to that returned in """
            """a Adzuna::API::Response::Location object.For example,"""
            """ "location0=UK&location1=South East England&location2=Surrey" """
            """ will performn a search over the county of Surrey."""
        ),
        field_type=FieldType.QueryParam,
    )
    location1: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    location2: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    location3: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    location4: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    location5: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    location6: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    location7: t.Optional[str] = Field(
        field_type=FieldType.QueryParam,
    )
    max_days_old: int = Field(
        None,
        description="The age of the oldest advertisment in days that will be returned.",
        field_type=FieldType.QueryParam,
    )
    category: str = Field(
        None,
        description='The category tag, as returned by the "category" endpoint.',
        field_type=FieldType.QueryParam,
    )
    sort_dir: SortDir = Field(
        None,
        description="The direction to order the search results.",
        field_type=FieldType.QueryParam,
    )
    sort_by: SortKey = Field(
        None,
        description="The ordering of the search results.",
        field_type=FieldType.QueryParam,
    )
    salary_min: int = Field(
        None,
        description="The minimum salary we wish to get results for.",
        field_type=FieldType.QueryParam,
    )
    salary_max: int = Field(
        None,
        description="The maximum salary we wish to get results for.",
        field_type=FieldType.QueryParam,
    )
    salary_include_unknown: Filter = Field(
        None,
        description="""If set it "1", jobs without a known salary are returned.""",
        field_type=FieldType.QueryParam,
    )
    full_time: Filter = Field(
        None,
        description="""If set to "1", only full time jobs will be returned.""",
        field_type=FieldType.QueryParam,
    )
    part_time: Filter = Field(
        None,
        description="""If set to "1", only part time jobs will be returned.""",
        field_type=FieldType.QueryParam,
    )
    contract: Filter = Field(
        None,
        description="""If set to "1", only contract jobs will be returned.""",
        field_type=FieldType.QueryParam,
    )
    permanent: Filter = Field(
        None,
        description="""If set to "1", only permanent jobs will be returned.""",
        field_type=FieldType.QueryParam,
    )
    company: str = Field(
        None,
        description=(
            "The canonical company name. This may be returned in a"
            " Adzuna::API::Response::Company object when a job is returned. A full list"
            " of allowed terms in not available through the API."
        ),
        field_type=FieldType.QueryParam,
    )


def read(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    params = parameters.dict()
    page = 1
    country = params.pop("country")

    while True:
        ADZUNA_JOBS_SEARCH_ENDPOINT = "{}/jobs/{}/search/{}".format(
            ADZUNA_ENDPOINT, country.value, page
        )
        response = requests.get(
            ADZUNA_JOBS_SEARCH_ENDPOINT,
            params=params,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to pull jobs from Adzuna params={}"
                " status_code={} response={}".format(
                    params, response.status_code, response.text
                )
            )
            raise Exception("Failed to pull jobs from Adzuna")

        jobs = response.json()["results"]
        for job in jobs:
            yield job

        page += 1
        if len(jobs) == 0 or page == MAX_NUMBER_OF_PAGES:
            break


AdzunaJobWarehouse = Warehouse(
    name="Adzuna Jobs",
    data_schema=AdzunaJob,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read,
        endpoints=[SEARCH_JOBS_ENDPOINT],
    ),
)



import base64
import typing as t
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.greenhouse.schemas import (
    GreenhouseJobModel,
    GreenhouseProfileModel,
)
from hrflow_connectors.core import (
    ActionEndpoints,
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)

GET_JOB_ENDPOINT = ActionEndpoints(
    name="Get job",
    description=(
        "Endpoint to get the content of a job with a given id."
        " The request method is `GET`"
    ),
    url="https://developers.greenhouse.io/harvest.html?shell#get-retrieve-job",
)
POST_CANDIDATE_ENDPOINT = ActionEndpoints(
    name="Post Candidate",
    description=(
        "Endpoint to create a new candidate and assign to a talent pool, the request"
        " method is `POST`"
    ),
    url="https://developers.greenhouse.io/job-board.html#jobs",
)


class WriteProfilesParameters(ParametersModel):
    auth: str = Field(..., description="XAPIKeyAuth", field_type=FieldType.Auth)
    on_behalf_of: str = Field(
        ...,
        description=(
            "The ID of the user sending the profile, or the person he is sending the"
            " profile on behalf of"
        ),
        field_type=FieldType.QueryParam,
    )


class ReadJobsParameters(ParametersModel):
    board_token: str = Field(
        ..., description="Board_token", field_type=FieldType.QueryParam
    )


def read(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    while True:
        response = requests.get(
            "https://boards-api.greenhouse.io/v1/boards/{}/jobs/?content=true".format(
                parameters.board_token
            ),
            headers={},
            params=None,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to pull jobs from Greenhouse params={}"
                " status_code={} response={}".format(
                    parameters, response.status_code, response.text
                )
            )
            raise Exception("Failed to pull jobs from Greenhouse")
        response = response.json()
        jobs = response["jobs"]
        if len(jobs) == 0:
            break

        adapter.info("Pulling {} jobs".format(len(jobs)))
        for job in jobs:
            yield job
        break


def write(
    adapter: LoggerAdapter,
    parameters: WriteProfilesParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    adapter.info("Pushing {} profiles".format(len(profiles)))
    failed_profiles = []
    for profile in profiles:
        parameters.auth = parameters.auth + ":"
        authorization = base64.b64encode(parameters.auth.encode("ascii"))
        response = requests.post(
            "https://harvest.greenhouse.io/v1/candidates",
            headers={
                "On-Behalf-Of": parameters.on_behalf_of,
                "Authorization": b"Basic " + authorization,
            },
            json=profile,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to push profile to Greenhouse"
                " status_code={} response={}".format(
                    response.status_code,
                    response.text,
                )
            )
            failed_profiles.append(profile)
    return failed_profiles


GreenhouseJobWarehouse = Warehouse(
    name="Greenhouse Jobs",
    data_schema=GreenhouseJobModel,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read,
        endpoints=[GET_JOB_ENDPOINT],
    ),
)

GreenhouseProfileWarehouse = Warehouse(
    name="Greenhouse Profiles",
    data_schema=GreenhouseProfileModel,
    data_type=DataType.profile,
    write=WarehouseWriteAction(
        parameters=WriteProfilesParameters,
        function=write,
        endpoints=[POST_CANDIDATE_ENDPOINT],
    ),
)



import typing as t
from enum import Enum
from logging import LoggerAdapter
from typing import List

import requests
from pydantic import BaseModel, Field

from hrflow_connectors.connectors.hubspot.schemas import ContactObject
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)

BASE_URL = "https://api.hubapi.com/crm/v3"


class Properties(str, Enum):  # more properrties can be added later to enrich
    COMPANY = "company"
    EMAIL = "email"
    FIRSTNAME = "firstname"
    LASTNAME = "lastname"
    PHONE = "phone"
    WEBSITE = "website"


class ReadProfilesParameters(ParametersModel):
    access_token: str = Field(
        ...,
        description=(
            "The token used to authenticate any API calls made for to your HubSpot"
            " account."
        ),
        repr=False,
        field_type=FieldType.Auth,
    )
    limit: int = Field(
        None,
        description="The maximum number of results to display per page.",
        field_type=FieldType.QueryParam,
    )
    after: str = Field(
        None,
        description=(
            "The paging cursor token of the last successfully read resource will be"
            " returned as the `paging.next.after` JSON property of a paged response"
            " containing more results."
        ),
        field_type=FieldType.QueryParam,
    )
    properties: List[Properties] = Field(
        None,
        description=(
            "A comma separated list of the properties to be returned in the response."
            " If any of the specified properties are not present on the requested"
            " object(s), they will be ignored."
        ),
        field_type=FieldType.QueryParam,
    )
    propertiesWithHistory: List[Properties] = Field(
        None,
        description=(
            "A comma separated list of the properties to be returned along with their"
            " history of previous values. If any of the specified properties are not"
            " present on the requested object(s), they will be ignored. Usage of this"
            " parameter will reduce the maximum number of objects that can be read by a"
            " single request."
        ),
        field_type=FieldType.QueryParam,
    )
    associations: List[str] = Field(
        None,
        description=(
            "A comma separated list of object types to retrieve associated IDs for. If"
            " any of the specified associations do not exist, they will be ignored."
        ),
        field_type=FieldType.QueryParam,
    )
    archived: bool = Field(
        default=False,
        description="Whether to return only results that have been archived.",
        field_type=FieldType.QueryParam,
    )


class Stage(BaseModel):
    label: str
    displayOrder: int
    metadata: str


class Pipeline(BaseModel):
    id: int
    label: str
    displayOrder: int
    stages: t.List[Stage]


class WriteProfilesParameters(ParametersModel):
    access_token: str = Field(
        ...,
        description=(
            "The token used to authenticate any API calls made for to your HubSpot"
            " account."
        ),
        repr=False,
        field_type=FieldType.Auth,
    )
    dealID: t.Optional[int] = Field(
        None,
        field_type=FieldType.QueryParam,
    )
    ticketID: t.Optional[int] = Field(
        None,
        field_type=FieldType.QueryParam,
    )
    pipeline: t.Optional[Pipeline] = Field(
        None,
        field_type=FieldType.QueryParam,
    )


# TODO: Improve how the contact's profile info is collected individually,
# take into account paggination
def read(
    adapter: LoggerAdapter,
    parameters: ReadProfilesParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    url = "{}/objects/contacts".format(BASE_URL)
    params = parameters.dict()
    del params["access_token"]

    response = requests.get(
        url,
        headers={"Authorization": "Bearer {}".format(parameters.access_token)},
        params=params,
    )
    if response.status_code // 100 != 2:
        adapter.error(
            "Failed to pull contacts list from Hubspot params={}"
            " status_code={} response={}".format(
                params, response.status_code, response.text
            )
        )
        raise Exception("Failed to pull contacts list from Hubspot")
    contacts = response.json()["results"]
    for contact in contacts:
        yield contact


# TODO: custom pipelines using the endpoint
# and add the contact to it using associations endpoint
def write(
    adapter: LoggerAdapter,
    parameters: WriteProfilesParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    adapter.info("Adding {} profiles to Hubspot ".format(len(profiles)))
    failed_profiles = []
    contacts_endpoint = "{}/objects/contacts".format(BASE_URL)
    for profile in profiles:
        response = requests.post(
            contacts_endpoint,
            headers={
                "Authorization": "Bearer {}".format(parameters.access_token),
            },
            json=profile,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to add profile to Hubspot status_code={} response={}".format(
                    response.status_code,
                    response.text,
                )
            )
            failed_profiles.append(profile)
        else:
            contactId = response.json()["id"]
            adapter.info("Successfully added contact with ID={}".format(contactId))
            dealId = parameters.dealID
            if dealId is not None:
                deals_endpoint = (
                    "{}/objects/deals/{}/associations/contacts/{}/3".format(
                        BASE_URL, dealId, contactId
                    )
                )
                headers = {
                    "Authorization": "Bearer {}".format(parameters.access_token),
                }
                deal_response = requests.request(
                    "PUT", deals_endpoint, headers=headers, data={}
                )
                if deal_response.status_code // 100 != 2:
                    adapter.error(
                        "Failed to add profile to Hubspot status_code={} response={}"
                        .format(
                            deal_response.status_code,
                            deal_response.text,
                        )
                    )
    return failed_profiles


HubspotContactWarehouse = Warehouse(
    name="Hubspot Contacts",
    data_schema=ContactObject,
    data_type=DataType.profile,
    read=WarehouseReadAction(parameters=ReadProfilesParameters, function=read),
    write=WarehouseWriteAction(parameters=WriteProfilesParameters, function=write),
)



import typing as t

from pydantic import BaseModel


class jobologyEventObject(BaseModel):
    type: str
    jobkey: t.Optional[str]
    firstName: t.Optional[str]
    lastName: t.Optional[str]
    phone: t.Optional[str]
    email: str
    cvUrl: str
    coverText: t.Optional[str]
    profilecountry: t.Optional[str]
    profileregions: t.Optional[str]
    profiledomains: t.Optional[str]
    joblien_annonce_site_carriere: t.Optional[str]
    statisticsource: t.Optional[str]
    statisticjbsource: t.Optional[str]



import typing as t
from enum import Enum
from logging import LoggerAdapter

import requests
from pydantic import Field

from hrflow_connectors.connectors.recruitee.schemas import (
    RecruiteeJob,
    RecruiteeProfile,
)
from hrflow_connectors.core import (
    DataType,
    FieldType,
    ParametersModel,
    ReadMode,
    Warehouse,
    WarehouseReadAction,
    WarehouseWriteAction,
)


class Sort(str, Enum):
    BY_DATE = "by_date"
    BY_LAST_MESSAGE = "by_last_message"


class Endpoint(str, Enum):
    STAGING_ENDPOINT = "STAGING ENDPOINT"
    PRODUCTION_ENDPOINT = "PRODUCTION ENDPOINT"


ENDPOINT_MAPPING = {
    Endpoint.STAGING_ENDPOINT: "https://api.s.recruitee.com/c",
    Endpoint.PRODUCTION_ENDPOINT: "https://api.recruitee.com/c",
}


class ReadProfilesParameters(ParametersModel):
    company_id: str = Field(
        ...,
        description="Company ID. A company subdomain can also be used.",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_token: str = Field(
        ...,
        description=(
            "Personal API Token allowing access to the Recruitee API from external"
            " services."
        ),
        repr=False,
        field_type=FieldType.Auth,
    )
    recruitee_endpoint: Endpoint = Field(
        ...,
        description="Specifies which endpoint to be used, satging or production.",
        field_type=FieldType.Other,
    )
    limit: int = Field(
        None,
        description="Specifies the number of candidates to retrieve",
        field_type=FieldType.QueryParam,
    )
    offset: int = Field(
        None,
        description=(
            "Skip number of candidates from the begining, used for ‘load more’, offset"
            " for next page should be current offset + limit"
        ),
        field_type=FieldType.QueryParam,
    )
    created_after: str = Field(
        None,
        description="Show only candidates created after given date",
        field_type=FieldType.QueryParam,
    )
    disqualified: bool = Field(
        None,
        description=(
            """Show only disqualified candidates who are disqualified in at least"""
            """ one job (should be string ‘true’ or ‘1’)."""
        ),
        field_type=FieldType.QueryParam,
    )
    qualified: bool = Field(
        None,
        description=(
            "Show only disqualified candidates who are qualified in at least one job"
            " (should be string ‘true’ or ‘1’)."
        ),
        field_type=FieldType.QueryParam,
    )
    ids: str = Field(
        None,
        description="List of IDs separated by comma, example: 234221,4211412,535432",
        field_type=FieldType.QueryParam,
    )
    offer_id: str = Field(
        None,
        description="Filter by offer",
        field_type=FieldType.QueryParam,
    )
    query: str = Field(
        None,
        description="Search query for candidate’s name or offer",
        field_type=FieldType.QueryParam,
    )
    sort: Sort = Field(
        None,
        description="Sorting options: by_date, by_last_message",
        field_type=FieldType.QueryParam,
    )
    with_messages: bool = Field(
        None,
        description=(
            "Show only candidates with messages (should be string ‘true’ or ‘1’)"
        ),
        field_type=FieldType.QueryParam,
    )
    with_my_messages: bool = Field(
        None,
        description=(
            "Show only candidates with messages that current admin sent (should be"
            " string ‘true’ or ‘1’"
        ),
        field_type=FieldType.QueryParam,
    )


class WriteProfilesParameters(ParametersModel):
    company_id: str = Field(
        ...,
        description="Company ID. A company subdomain can also be used.",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_token: str = Field(
        ...,
        description=(
            "Personal API Token allowing access to the Recruitee API from external"
            " services."
        ),
        repr=False,
        field_type=FieldType.Auth,
    )
    recruitee_endpoint: Endpoint = Field(
        ...,
        description="Specifies which endpoint to be used, satging or production.",
        field_type=FieldType.Other,
    )
    offer_ids: t.List[int] = Field(
        None,
        description=(
            "Offers to which the candidate will be assigned with default stage. You can"
            " also pass one ID as offer_id"
        ),
        field_type=FieldType.QueryParam,
    )


class View_mode(str, Enum):
    DEFAULT = "default"
    BRIEF = "brief"


class ReadJobsParameters(ParametersModel):
    company_id: str = Field(
        ...,
        description="Company ID. A company subdomain can also be used.",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_token: str = Field(
        ...,
        description=(
            "Personal API Token allowing access to the Recruitee API from external"
            " services."
        ),
        repr=False,
        field_type=FieldType.Auth,
    )
    recruitee_endpoint: Endpoint = Field(
        ...,
        description="Specifies which endpoint to be used, satging or production.",
        field_type=FieldType.Other,
    )

    kind: str = Field(
        None,
        description=(
            "If no kind is given, returns all job offers, if kind is job then lists"
            " only jobs, if scope is talent_pool, lists only talent pools"
        ),
        field_type=FieldType.QueryParam,
    )
    scope: str = Field(
        None,
        description=(
            "If no scope is given list all job offers. archived returns only archived"
            " job offers, active returns published, internal and closed job offers,"
            " not_archived returns all but archived jobs"
        ),
        field_type=FieldType.QueryParam,
    )
    view_mode: View_mode = Field(
        default="brief",
        description=(
            "default (default mode, includes most of offer details); brief (only"
            " offer’s id, title, status and kind)"
        ),
        field_type=FieldType.QueryParam,
    )


class WriteJobsParameters(ParametersModel):
    company_id: str = Field(
        ...,
        description="Company ID. A company subdomain can also be used.",
        repr=False,
        field_type=FieldType.Auth,
    )
    api_token: str = Field(
        ...,
        description=(
            "Personal API Token allowing access to the Recruitee API from external"
            " services."
        ),
        repr=False,
        field_type=FieldType.Auth,
    )
    recruitee_endpoint: Endpoint = Field(
        ...,
        description="Specifies which endpoint to be used, satging or production.",
        field_type=FieldType.Other,
    )


def read_candidates(
    adapter: LoggerAdapter,
    parameters: ReadProfilesParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    RECRUITEE_ENDPOINT = ENDPOINT_MAPPING[parameters.recruitee_endpoint]
    params = parameters.dict()
    del params["api_token"]
    del params["company_id"]
    del params["recruitee_endpoint"]

    response = requests.get(
        "{}/{}/candidates".format(RECRUITEE_ENDPOINT, parameters.company_id),
        headers={
            "Authorization": "Bearer {}".format(parameters.api_token),
        },
        params=params,
    )
    if response.status_code // 100 != 2:
        adapter.error(
            "Failed to pull candidates list from Recruitee params={}"
            " status_code={} response={}".format(
                params, response.status_code, response.text
            )
        )
    candidates = response.json()["candidates"]
    for candidate in candidates:
        yield candidate


def write_candidates(
    adapter: LoggerAdapter,
    parameters: WriteProfilesParameters,
    profiles: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    adapter.info("Adding {} profiles to Recruitee ".format(len(profiles)))
    RECRUITEE_ENDPOINT = ENDPOINT_MAPPING[parameters.recruitee_endpoint]
    failed_profiles = []
    for profile in profiles:
        payload = {"candidate": profile, "offers": parameters.offer_ids}

        response = requests.post(
            "{}/{}/candidates".format(RECRUITEE_ENDPOINT, parameters.company_id),
            headers={
                "Accept": "application/json",
                "Content-Type": "application/json",
                "Authorization": "Bearer {}".format(parameters.api_token),
            },
            json=payload,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to add candidate to Recruitee status_code={} response={}"
                .format(
                    response.status_code,
                    response.text,
                )
            )
            failed_profiles.append(profile)
    return failed_profiles


def read_jobs(
    adapter: LoggerAdapter,
    parameters: ReadJobsParameters,
    read_mode: t.Optional[ReadMode] = None,
    read_from: t.Optional[str] = None,
) -> t.Iterable[t.Dict]:
    RECRUITEE_ENDPOINT = ENDPOINT_MAPPING[parameters.recruitee_endpoint]
    params = dict(
        kind=parameters.kind,
        scope=parameters.scope,
        view_mode=parameters.view_mode,
    )

    response = requests.get(
        "{}/{}/offers".format(RECRUITEE_ENDPOINT, parameters.company_id),
        headers={"Authorization": "Bearer {}".format(parameters.api_token)},
        params=params,
    )
    if response.status_code // 100 != 2:
        adapter.error(
            "Failed to pull jobs from Recruitee params={}"
            " status_code={} response={}".format(
                params, response.status_code, response.text
            )
        )
        raise Exception("Failed to pull jobs from Recruitee")
    jobs = response.json()["offers"]
    for job in jobs:
        full_job_response = requests.get(
            "{}/{}/offers/{}".format(
                RECRUITEE_ENDPOINT, parameters.company_id, job["id"]
            ),
            headers={"Authorization": "Bearer {}".format(parameters.api_token)},
        )
        if full_job_response.status_code // 100 != 2:
            adapter.error(
                "Failed to pull jobs details from Recruitee job_id={}"
                " status_code={} response={}".format(
                    job["id"],
                    full_job_response.status_code,
                    full_job_response.text,
                )
            )
            raise Exception("Failed to pull jobs from Recruitee")
        yield full_job_response.json()["offer"]


def write_jobs(
    adapter: LoggerAdapter,
    parameters: WriteJobsParameters,
    jobs: t.Iterable[t.Dict],
) -> t.List[t.Dict]:
    adapter.info("Adding {} job offers to Recruitee ".format(len(jobs)))
    RECRUITEE_ENDPOINT = ENDPOINT_MAPPING[parameters.recruitee_endpoint]
    failed_jobs = []
    for job in jobs:
        payload = {"offer": job}
        response = requests.post(
            "{}/{}/offers".format(RECRUITEE_ENDPOINT, parameters.company_id),
            headers={
                "Accept": "application/json",
                "Content-Type": "application/json",
                "Authorization": "Bearer {}".format(parameters.api_token),
            },
            json=payload,
        )
        if response.status_code // 100 != 2:
            adapter.error(
                "Failed to add job offers to Recruitee status_code={} response={}"
                .format(
                    response.status_code,
                    response.text,
                )
            )
            failed_jobs.append(job)
    return failed_jobs


RecruiteeProfileWarehouse = Warehouse(
    name="Recruitee Profiles",
    data_schema=RecruiteeProfile,
    data_type=DataType.profile,
    read=WarehouseReadAction(
        parameters=ReadProfilesParameters,
        function=read_candidates,
    ),
    write=WarehouseWriteAction(
        parameters=WriteProfilesParameters,
        function=write_candidates,
    ),
)


RecruiteeJobWarehouse = Warehouse(
    name="Recruitee Jobs",
    data_schema=RecruiteeJob,
    data_type=DataType.job,
    read=WarehouseReadAction(
        parameters=ReadJobsParameters,
        function=read_jobs,
    ),
    write=WarehouseWriteAction(
        parameters=WriteJobsParameters,
        function=write_jobs,
    ),
)
